# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import coda
import copy
import wx
import os.path
import weakref


(TYPE_INT8, TYPE_UINT8, TYPE_INT16, TYPE_UINT16, TYPE_INT32, TYPE_UINT32, TYPE_INT64, TYPE_UINT64,
 TYPE_FLOAT32, TYPE_FLOAT64, TYPE_CHAR, TYPE_STRING, TYPE_BYTES, TYPE_NO_DATA, TYPE_VSF_INTEGER, TYPE_TIME,
 TYPE_COMPLEX, TYPE_ARRAY, TYPE_RECORD, TYPE_UNKNOWN) = list(range(20))

_CodaNativeTypeDictionary = {coda.coda_native_type_int8: TYPE_INT8,
                             coda.coda_native_type_uint8: TYPE_UINT8,
                             coda.coda_native_type_int16: TYPE_INT16,
                             coda.coda_native_type_uint16: TYPE_UINT16,
                             coda.coda_native_type_int32: TYPE_INT32,
                             coda.coda_native_type_uint32: TYPE_UINT32,
                             coda.coda_native_type_int64: TYPE_INT64,
                             coda.coda_native_type_uint64: TYPE_UINT64,
                             coda.coda_native_type_float: TYPE_FLOAT32,
                             coda.coda_native_type_double: TYPE_FLOAT64,
                             coda.coda_native_type_char: TYPE_CHAR,
                             coda.coda_native_type_string: TYPE_STRING,
                             coda.coda_native_type_bytes: TYPE_BYTES}

_CodaSpecialTypeDictionary = {coda.coda_special_no_data: TYPE_NO_DATA,
                              coda.coda_special_vsf_integer: TYPE_VSF_INTEGER,
                              coda.coda_special_time: TYPE_TIME,
                              coda.coda_special_complex: TYPE_COMPLEX}

_TypeDictionary = {TYPE_INT8: "int8",
                   TYPE_UINT8: "uint8",
                   TYPE_INT16: "int16",
                   TYPE_UINT16: "uint16",
                   TYPE_INT32: "int32",
                   TYPE_UINT32: "uint32",
                   TYPE_INT64: "int64",
                   TYPE_UINT64: "uint64",
                   TYPE_FLOAT32: "float",
                   TYPE_FLOAT64: "double",
                   TYPE_CHAR: "char",
                   TYPE_STRING: "string",
                   TYPE_BYTES: "bytes",
                   TYPE_NO_DATA: "no data",
                   TYPE_VSF_INTEGER: "vsf integer",
                   TYPE_TIME: "time",
                   TYPE_COMPLEX: "complex",
                   TYPE_ARRAY: "array",
                   TYPE_RECORD: "record",
                   TYPE_UNKNOWN: "unknown"}

icons = None
iconImageList = None
iconDictionary = None
expandedIconDictionary = None


class CorruptProductError(Exception):

    pass


class Node(object):

    def __init__(self, type=TYPE_UNKNOWN):
        self.parent = None
        self.cursor = None

        self.name = ""
        self.real_name = ""
        self.type = type

        # only record fields have an associated availability
        # for all other Nodes available is always True.
        self.available = True

        # only record fields can be hidden, for all other
        # Nodes hidden is always False.
        self.hidden = False

    def initialize(self):
        assert self.cursor, "cursor should be set before calling initialize()"

    def _path(self, path):
        path.append(self)

        if self.parent is None:
            path.reverse()
            return path
        else:
            return self.parent._path(path)

    def path(self):
        return self._path([])

    # we cache the bit size here. this is not very clean, because only
    # the CODA backend has meaningful bit sizes. however, requesting
    # a bit size for other backends is legal (result will be -1) and
    # it saves a large amount of time while navigating through a product.
    def getBitSize(self):
        assert self.cursor, "cursor should be set before calling getBitSize()"

        try:
            size = self._bitSize
        except AttributeError:
            # if an error occurs while trying to determine the
            # bit size, the _bitSize attribute is not added to the
            # Node instance. this is appropriate, because the bit size
            # is still unknown.
            size = coda.cursor_get_bit_size(self.cursor)
            self._bitSize = size

        return size


class StringNode(Node):

    def __init__(self):
        Node.__init__(self, TYPE_STRING)
        self._length = -1

    def initialize(self):
        Node.initialize(self)
        self._length = coda.cursor_get_string_length(self.cursor)

    def __len__(self):
        return self._length


class CompoundNode(Node):

    def __init__(self, type=TYPE_UNKNOWN):
        assert type in (TYPE_UNKNOWN, TYPE_RECORD, TYPE_ARRAY)
        Node.__init__(self, type)
        self._count = -1

    def initialize(self):
        Node.initialize(self)
        self._count = coda.cursor_get_num_elements(self.cursor)

    def __len__(self):
        return self._count


class RecordNode(CompoundNode):

    def __init__(self):
        CompoundNode.__init__(self, TYPE_RECORD)
        self._fields = None

    def dirty(self):
        return self._fields is None

    def __getitem__(self, key):
        if not isinstance(key, int):
            raise TypeError("index should be an integer")

        if self.dirty():
            raise ValueError("cannot retrieve field information from dirty record")

        if key < 0 or key > len(self._fields):
            raise IndexError("field index out of range")

        return self._fields[key]


class ArrayNode(CompoundNode):

    def __init__(self, base=TYPE_UNKNOWN):
        CompoundNode.__init__(self, TYPE_ARRAY)
        self.base = base
        self.dimensions = None

    def initialize(self):
        CompoundNode.initialize(self)
        self.dimensions = coda.cursor_get_array_dim(self.cursor)

    def isRankZero(self):
        return (len(self.dimensions) == 0)

    def isObjectArray(self):
        return isinstance(self, ObjectArrayNode)


class ObjectArrayNode(ArrayNode):

    def __init__(self, base=TYPE_UNKNOWN):
        ArrayNode.__init__(self, base)
        self.cache = {}

    def initialize(self):
        ArrayNode.initialize(self)

    def __getitem__(self, key):
        if not isinstance(key, int):
            raise TypeError("index should be an integer")

        if key < 0 or key > len(self):
            raise IndexError("array index out of range")

        return _RetrieveArrayElement(self, key)


def NodeFactory(cursor):
    nodeCodaType = coda.cursor_get_type(cursor)
    nodeCodaClass = coda.type_get_class(nodeCodaType)

    type = TranslateCodaType(nodeCodaClass, nodeCodaType)

    if type == TYPE_RECORD:
        return RecordNode()

    elif (type == TYPE_ARRAY):
        baseCodaType = coda.type_get_array_base_type(nodeCodaType)
        baseCodaClass = coda.type_get_class(baseCodaType)

        baseType = TranslateCodaType(baseCodaClass, baseCodaType)

        if ((baseType == TYPE_RECORD) or (baseType == TYPE_ARRAY)):
            return ObjectArrayNode(baseType)
        else:
            return ArrayNode(baseType)

    elif type == TYPE_STRING:
        return StringNode()

    else:
        return Node(type)


def TranslateCodaType(nodeClass, nodeType):
    if nodeClass == coda.coda_record_class:
        return TYPE_RECORD

    elif (nodeClass == coda.coda_array_class):
        return TYPE_ARRAY

    elif ((nodeClass == coda.coda_integer_class) or (nodeClass == coda.coda_real_class) or
          (nodeClass == coda.coda_text_class) or (nodeClass == coda.coda_raw_class)):

        # scalar type.
        nodeReadType = coda.type_get_read_type(nodeType)

        if nodeReadType == coda.coda_native_type_not_available:
            raise TypeError("TranslateCodaType():: coda_native_type_not_available encountered.")
        else:
            try:
                return _CodaNativeTypeDictionary[nodeReadType]
            except KeyError:
                return TYPE_UNKNOWN

    elif nodeClass == coda.coda_special_class:
        # special type.
        nodeSpecialType = coda.type_get_special_type(nodeType)

        try:
            return _CodaSpecialTypeDictionary[nodeSpecialType]
        except KeyError:
            return TYPE_UNKNOWN
    else:
        return TYPE_UNKNOWN


def RetrieveFieldInfo(node):
    assert node.dirty(), "_RetrieveFieldInfo() should only be called on dirty nodes."

    node._fields = []

    if len(node) > 0:
        try:
            cursor = copy.deepcopy(node.cursor)
            nodeCodaType = coda.cursor_get_type(node.cursor)
            coda.cursor_goto_first_record_field(cursor)
            for i in range(0, len(node)):
                field = NodeFactory(cursor)
                node._fields.append(field)
                field.parent = weakref.proxy(node)
                field.cursor = copy.deepcopy(cursor)
                field.name = coda.type_get_record_field_name(nodeCodaType, i)
                field.real_name = coda.type_get_record_field_real_name(nodeCodaType, i)
                field.available = (coda.cursor_get_record_field_available_status(node.cursor, i) == 1)
                field.hidden = (coda.type_get_record_field_hidden_status(nodeCodaType, i) == 1)
                field.initialize()

                if i < len(node) - 1:
                    coda.cursor_goto_next_record_field(cursor)
        except coda.CodacError as ex:
            # restore state
            node._fields = None
            raise CorruptProductError("[CODA] %s" % (str(ex),))


def _RetrieveArrayElement(node, index):
    assert index >= 0, "Trying to retrieve element at unspecified index (-1)"

    # try to retrieve the current element from the cache.
    try:
        return node.cache[index]
    except KeyError:
        # cache miss.
        pass

    # optimisation suggested by Sander.
    previousElement = None
    if index > 0:
        try:
            previousElement = node.cache[index - 1]
        except KeyError:
            # cache miss.
            pass

    # try to retrieve the node
    try:
        if previousElement:
            cursor = copy.deepcopy(previousElement.cursor)
            # goto_next_array_element is much faster than goto_array_element
            # for large arrays.
            coda.cursor_goto_next_array_element(cursor)
        else:
            cursor = copy.deepcopy(node.cursor)
            coda.cursor_goto_array_element_by_index(cursor, index)

        element = NodeFactory(cursor)
        element.parent = weakref.proxy(node)
        element.cursor = cursor
        element.name = GetMultiDimensionalIndex(node, index)
        element.initialize()
    except coda.CodacError as ex:
        raise CorruptProductError("[CODA] %s" % (str(ex),))

    # cache element
    node.cache[index] = element
    return element


def RetrieveAttributes(node):
    cursor = copy.deepcopy(node.cursor)

    try:
        coda.cursor_goto_attributes(cursor)
        attributes = NodeFactory(cursor)
        attributes.cursor = cursor
        attributes.name = "attributes"
        attributes.initialize()
        return attributes

    except coda.CodacError as ex:
        raise CorruptProductError("[CODA] %s" % (str(ex),))


def GetMultiDimensionalIndex(node, index):
    if node.isRankZero():
        assert index == 0, "Rank-0 array can only be indexed with 0"
        return [0]
    else:
        multiDimensionalIndex = []
        for i in reversed(list(range(1, len(node.dimensions)))):
            multiDimensionalIndex.append(index % node.dimensions[i])
            index -= index % node.dimensions[i]
            index /= node.dimensions[i]
        multiDimensionalIndex.append(index)
        multiDimensionalIndex.reverse()
        return multiDimensionalIndex


def GetNodeAsListItem(index, node, data=None):
    item = wx.ListItem()
    if data:
        item.SetMask(wx.LIST_MASK_TEXT | wx.LIST_MASK_IMAGE | wx.LIST_MASK_DATA)
        item.SetData(data)
    else:
        item.SetMask(wx.LIST_MASK_TEXT | wx.LIST_MASK_IMAGE)

    item.SetId(index)
    item.SetText(node.name)
    item.SetImage(GetNodeIcon(node, iconDictionary))

    if node.hidden:
        item.SetFont(item.GetFont().Italic())

    if (not node.available) or (isinstance(node, CompoundNode) and len(node) == 0):
        item.SetTextColour("light grey")

    return item


def GetTypeAsString(type):
    try:
        return _TypeDictionary[type]
    except KeyError:
        return "unknown"


def GetDataAsString(type, data, maxLength=-1):
    assert data is not None, "\"data\" argument should not be None."

    if type == TYPE_STRING or type == TYPE_CHAR:
        if maxLength >= 0 and len(data) > maxLength:
            return "<string of length %u>" % (len(data),)
        else:
            return "\"%s\"" % (data,)

    elif type == TYPE_TIME:
        try:
            return coda.time_to_string(data)
        except coda.CodacError:
            return "<invalid time value>"

    elif type == TYPE_BYTES:
        # convert length in bytes to length in characters ("0x%x " per byte).
        if maxLength >= 0 and (data.size * 5) > maxLength:
            return "<data block of %u byte(s)>" % (data.size,)
        elif data.size > 0:
            result = ""
            for byte in data[:-1]:
                result += "0x%x " % (byte,)
            result += "0x%x" % (data[-1],)
            return result
        else:
            return ""

    elif type == TYPE_RECORD:
        # return a key-value list of the fields between braces
        if len(data) == 0:
            return "<empty record>"
        result = "("
        for field in data._registeredFields:
            result += "%s=%s, " % (field, str(data.__dict__[field]),)
        result = result[:-2] + ")"
        if maxLength >= 0 and len(result) > maxLength:
            return "<record with %u field%s>" % (len(data), "s" if len(data) != 1 else "")
        return result

    else:
        return str(data)


def GetPathAsString(path):
    if len(path) == 0:
        return "/"
    else:
        pathString = "/"
        for place in path[:-1]:
            pathString += str(place.name)
            if place.type == TYPE_RECORD and pathString != "/":
                pathString += "/"
        pathString += str(path[-1].name)
        return pathString


def GetNodeIcon(node, dict):
    if not dict:
        return 0
    try:
        return dict[node.type]
    except KeyError:
        return 0


# code to initialize a 'standard icon library'
def InitIcons():
    global icons, iconImageList, iconDictionary, expandedIconDictionary

    if not icons:
        iconDir = wx.Config.Get().Read("DirectoryLocation/ApplicationData")
        icons = []
        iconImageList = wx.ImageList(16, 16)

        tmp = wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, (16, 16))
        icons.append(tmp)
        ICON_DEFAULT = iconImageList.Add(tmp)

        tmp = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE, wx.ART_OTHER, (16, 16))
        icons.append(tmp)
        ICON_UNAVAILABLE = iconImageList.Add(tmp)

        tmp = wx.ArtProvider.GetBitmap(wx.ART_FOLDER, wx.ART_OTHER, (16, 16))
        icons.append(tmp)
        ICON_FOLDER = iconImageList.Add(tmp)

        tmp = wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN, wx.ART_OTHER, (16, 16))
        icons.append(tmp)
        ICON_OPEN_FOLDER = iconImageList.Add(tmp)

        tmp = wx.Icon(os.path.join(iconDir, "array.ico"), wx.BITMAP_TYPE_ICO)
        icons.append(tmp)
        ICON_ARRAY = iconImageList.Add(tmp)

        iconDictionary = {TYPE_ARRAY: ICON_ARRAY,
                          TYPE_NO_DATA: ICON_UNAVAILABLE,
                          TYPE_RECORD: ICON_FOLDER}

        expandedIconDictionary = {TYPE_RECORD: ICON_OPEN_FOLDER}



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx

from .preferences import PreferencesDialog
from .shell import VisanShell


class VisanFrame(wx.Frame):

    def __init__(self, app, title, pos, size):
        wx.Frame.__init__(self, None, -1, title, pos, size)

        self.app = app
        self.closingdown = False

        if wx.Config.Get().Read("IconFile"):
            self.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))

        # Create and configure all widgets
        self.CreateMenuBar()
        self.CreateStatusBar()
        self.shell = VisanShell(self)
        self.shell.SetFocus()

        # Other Event Listeneres
        self.Bind(wx.EVT_CLOSE, self.OnExit)

    def CreateMenuBar(self):
        menuBar = wx.MenuBar()

        filemenu = wx.Menu()

        item = filemenu.Append(wx.ID_OPEN, "&Load and Execute Script...\tCtrl+O",
                               "Load and execute a VISAN script from a file")
        self.Bind(wx.EVT_MENU, self.OnExecuteScript, item)

        recentmenu = wx.Menu()

        self.recentMenuItem = filemenu.Append(wx.ID_ANY, "&Recent Scripts", recentmenu)
        self.recentMenuItem.Enable(False)
        self.app.filehistory.UseMenu(recentmenu)
        self.Bind(wx.EVT_MENU_RANGE, self.OnFileHistory, id=wx.ID_FILE1, id2=wx.ID_FILE9)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_ANY, "&Browse Product...\tCtrl+B", "Browse the contents of a product file")
        self.Bind(wx.EVT_MENU, self.OnBrowseProduct, item)

        item = filemenu.Append(wx.ID_ANY, "&Harp Import...\tCtrl+I", "Import a product file using HARP")
        self.Bind(wx.EVT_MENU, self.OnHarpImport, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_ANY, "&Show Log", "Show the contents of the console log")
        self.Bind(wx.EVT_MENU, self.OnShowLog, item)

        item = filemenu.Append(wx.ID_ANY, "&Clear Log", "Clear the console log")
        self.Bind(wx.EVT_MENU, self.OnClearLog, item)

        item = filemenu.Append(wx.ID_SAVE, "S&ave Log to script...\tCtrl+S", "Save the console log as a VISAN script")
        self.Bind(wx.EVT_MENU, self.OnSaveLog, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_PREFERENCES, "&Preferences...", "Edit Preferences")
        self.Bind(wx.EVT_MENU, self.OnPrefs, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_CLOSE, "&Close\tCtrl-W", "Close this Window (and terminate the application)")
        self.Bind(wx.EVT_MENU, self.OnExit, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_EXIT, "E&xit VISAN\tCtrl+Q", "Terminate the application")
        self.Bind(wx.EVT_MENU, self.OnExit, item)

        menuBar.Append(filemenu, "&File")

        editmenu = wx.Menu()

        item = editmenu.Append(wx.ID_UNDO, "&Undo \tCtrl+Z", "Undo the last action")
        self.Bind(wx.EVT_MENU, self.OnUndo, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_UNDO)

        item = editmenu.Append(wx.ID_REDO, "&Redo \tCtrl+Y", "Redo the last undone action")
        self.Bind(wx.EVT_MENU, self.OnRedo, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_REDO)

        editmenu.AppendSeparator()

        item = editmenu.Append(wx.ID_CUT, "Cu&t \tCtrl+X", "Cut the selection")
        self.Bind(wx.EVT_MENU, self.OnCut, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_CUT)

        item = editmenu.Append(wx.ID_COPY, "&Copy \tCtrl+C", "Copy the selection")
        self.Bind(wx.EVT_MENU, self.OnCopy, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_COPY)

        item = editmenu.Append(wx.ID_PASTE, "&Paste \tCtrl+V", "Paste from clipboard")
        self.Bind(wx.EVT_MENU, self.OnPaste, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_PASTE)

        item = editmenu.Append(wx.ID_CLEAR, "&Delete", "Delete the selection")
        self.Bind(wx.EVT_MENU, self.OnDelete, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_CLEAR)

        editmenu.AppendSeparator()

        item = editmenu.Append(wx.ID_SELECTALL, "Select A&ll \tCtrl+A", "Select all text")
        self.Bind(wx.EVT_MENU, self.OnSelectAll, item)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateMenu, id=wx.ID_SELECTALL)

        menuBar.Append(editmenu, "&Edit")

        helpmenu = wx.Menu()

        item = helpmenu.Append(wx.ID_HELP, "&VISAN Documentation\tF1", "Display VISAN Documentation")
        self.Bind(wx.EVT_MENU, self.OnVisanHelp, item)

        item = helpmenu.Append(wx.ID_ANY, "&CODA Documentation", "Display CODA Documentation")
        self.Bind(wx.EVT_MENU, self.OnCodaHelp, item)

        item = helpmenu.Append(wx.ID_ANY, "&HARP Documentation", "Display HARP Documentation")
        self.Bind(wx.EVT_MENU, self.OnHarpHelp, item)

        helpmenu.AppendSeparator()

        item = helpmenu.Append(wx.ID_ANY, "&CODA Product Format Definitions\tCtrl-F1",
                               "Display Product Format Specification Documentation for products supported by CODA")
        self.Bind(wx.EVT_MENU, self.OnCodaDefHelp, item)

        helpmenu.AppendSeparator()

        item = helpmenu.Append(wx.ID_ANY, "&Python Documentation", "Display Python Documentation")
        self.Bind(wx.EVT_MENU, self.OnPythonHelp, item)

        item = helpmenu.Append(wx.ID_ANY, "&Numpy Documentation", "Display Numpy Documentation")
        self.Bind(wx.EVT_MENU, self.OnNumpyHelp, item)

        helpmenu.AppendSeparator()

        item = helpmenu.Append(wx.ID_ANY, "&Getting Started with VISAN", "Display a short VISAN introduction")
        self.Bind(wx.EVT_MENU, self.OnIntro, item)

        item = helpmenu.Append(wx.ID_ABOUT, "&About VISAN",
                               "Display the VISAN splash screen, version number and credits")
        self.Bind(wx.EVT_MENU, self.OnAbout, item)

        menuBar.Append(helpmenu, "&Help")

        self.SetMenuBar(menuBar)

        if wx.Config.Get().Read('UserMode') == "Developer":

            developmenu = wx.Menu()

            item = developmenu.Append(wx.ID_ANY, "Open VISAN &Shell", "Open a VISAN/Python command line shell")
            self.Bind(wx.EVT_MENU, self.OnShell, item)

            item = developmenu.Append(wx.ID_ANY, "Open VISAN &Crust", "Open a VISAN/Python enhanced command line shell")
            self.Bind(wx.EVT_MENU, self.OnCrust, item)

            menuBar.Append(developmenu, "[&Develop]")

    def OnPrefs(self, event):
        PreferencesDialog(self).ShowModal()

    def OnExit(self, event):
        if self.closingdown:
            return

        self.closingdown = True
        for i in self.GetChildren():
            if isinstance(i, wx.Frame):
                i.Close(True)

        self.Destroy()
        self.app.Exit()

    def OnShowLog(self, event):
        self.app.ShowLog()

    def OnClearLog(self, event):
        self.app.ClearLog()

    def OnSaveLog(self, event):
        self.app.SaveLog()

    def OnExecuteScript(self, event):
        self.app.ExecuteScript()

    def OnBrowseProduct(self, event):
        self.app.BrowseProduct()

    def OnHarpImport(self, event):
        self.app.HarpImport()

    def OnFileHistory(self, event):
        filenum = event.GetId() - wx.ID_FILE1
        self.app.FileHistory(filenum)

    def OnShowOzone(self, event):
        self.app.ShowOzone()

    def OnIntro(self, event):
        self.app.ShowIntro()

    def OnAbout(self, event):
        self.app.ShowAbout()

    def OnVisanHelp(self, event):
        self.app.ShowHelp(mode="mainindex")

    def OnCodaHelp(self, event):
        self.app.ShowHelp(mode="coda")

    def OnHarpHelp(self, event):
        self.app.ShowHelp(mode="harp")

    def OnCodaDefHelp(self, event):
        self.app.ShowHelp(mode="codadef")

    def OnPythonHelp(self, event):
        self.app.ShowHelp(mode="python")

    def OnNumpyHelp(self, event):
        self.app.ShowHelp(mode="numpy")

    def OnShell(self, event):
        self.app.OpenShell()

    def OnCrust(self, event):
        self.app.OpenCrust()

    def OnUndo(self, event):
        win = wx.Window.FindFocus()
        win.Undo()

    def OnRedo(self, event):
        win = wx.Window.FindFocus()
        win.Redo()

    def OnCut(self, event):
        win = wx.Window.FindFocus()
        win.Cut()

    def OnCopy(self, event):
        win = wx.Window.FindFocus()
        win.Copy()

    def OnPaste(self, event):
        win = wx.Window.FindFocus()
        win.Paste()

    def OnDelete(self, event):
        win = wx.Window.FindFocus()
        win.Clear()

    def OnSelectAll(self, event):
        win = wx.Window.FindFocus()
        win.SelectAll()

    def OnUpdateMenu(self, event):
        win = wx.Window.FindFocus()
        eventid = event.GetId()
        event.Enable(True)
        try:
            if eventid == wx.ID_UNDO:
                event.Enable(win.CanUndo())
            elif eventid == wx.ID_REDO:
                event.Enable(win.CanRedo())
            elif eventid == wx.ID_CUT:
                event.Enable(win.CanCut())
            elif eventid == wx.ID_COPY:
                event.Enable(win.CanCopy())
            elif eventid == wx.ID_PASTE:
                event.Enable(win.CanPaste())
            elif eventid == wx.ID_CLEAR:
                event.Enable(win.CanCut())
            elif eventid == wx.ID_SELECTALL:
                event.Enable(hasattr(win, 'SelectAll'))
            else:
                event.Enable(False)
        except AttributeError:
            # This menu option is not supported in the current context.
            event.Enable(False)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx


class HarpImportDialog(wx.Dialog):

    def __init__(self, *args, **kwargs):
        wx.Dialog.__init__(self, *args, **kwargs)
        self.InitUI()
        self.SetTitle("Import product using Harp")

    def InitUI(self):
        panel = wx.Panel(self)
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        grid = wx.GridBagSizer(hgap=5, vgap=5)

        self.fileLabel = wx.StaticText(self, label="file path :")
        grid.Add(self.fileLabel, flag=wx.ALIGN_RIGHT | wx.ALL, pos=(0, 0))
        self.filePicker = wx.FilePickerCtrl(self, size=(500, -1), style=wx.FLP_USE_TEXTCTRL)
        grid.Add(self.filePicker, pos=(0, 1))

        self.operationsLabel = wx.StaticText(self, label="operations :")
        grid.Add(self.operationsLabel, flag=wx.ALIGN_RIGHT | wx.ALL, pos=(1, 0))
        self.operationsText = wx.TextCtrl(self, size=(500, -1))
        self.operationsText.SetToolTip(wx.ToolTip("semi-colon separated string of operations to apply as part of "
                                                  "the import (see HARP documentation for further details)"))
        grid.Add(self.operationsText, pos=(1, 1))

        self.ingestionOptionsLabel = wx.StaticText(self, label="ingestion options :")
        grid.Add(self.ingestionOptionsLabel, flag=wx.ALIGN_RIGHT | wx.ALL, pos=(2, 0))
        self.ingestionOptionsText = wx.TextCtrl(self, size=(500, -1))
        self.ingestionOptionsText.SetToolTip(wx.ToolTip("semi-colon separated string of key=value pairs to use as "
                                                        "ingestion options when importing from a product that is not "
                                                        "in HARP format (see HARP 'ingestion definitions' "
                                                        "documentation for further details)"))
        grid.Add(self.ingestionOptionsText, pos=(2, 1))

        self.variableLabel = wx.StaticText(self, label="variable name :")
        grid.Add(self.variableLabel, flag=wx.ALIGN_RIGHT | wx.ALL, pos=(3, 0))
        self.variableText = wx.TextCtrl(self, value="product", size=(200, -1))
        self.variableText.SetToolTip(wx.ToolTip("the name of the variable as it will end up in the VISAN Python shell"))
        grid.Add(self.variableText, pos=(3, 1))

        mainSizer.Add(grid, 0, wx.ALL, 5)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        okButton = wx.Button(self, wx.ID_OK, label='Import')
        cancelButton = wx.Button(self, wx.ID_CANCEL, label='Cancel')
        hbox.Add(okButton)
        hbox.Add(cancelButton, flag=wx.LEFT, border=5)
        mainSizer.Add(hbox, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        self.SetSizer(mainSizer)
        mainSizer.Fit(self)

    def OnHarpFormat(self, event):
        self.ingestionOptionsLabel.Enable(not self.harpformatCtrl.IsChecked())
        self.ingestionOptionsText.Enable(not self.harpformatCtrl.IsChecked())

    def GetFilename(self):
        return self.filePicker.GetPath()

    def GetIsHarpFormat(self):
        return self.harpformatCtrl.IsChecked()

    def GetOperations(self):
        return self.operationsText.GetValue()

    def GetIngestionOptions(self):
        return self.ingestionOptionsText.GetValue()

    def GetVariableName(self):
        return self.variableText.GetValue()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
Custom dialog for allowing the user to easily specify a filename
of a certain type, that is: a name with a verified extension.

The use provides the TypedSaveFileDialog upon creation with a
list of (description, extension) tuples representing the allowed
file types.

The Dialog will only successfully exit if the user specifies a
filename that is of an allowed type, either by specifying a
filename that ends in one of the supplied extensions, or by
explicitly connecting one of the supplied types to an arbitrary
filename by choosing a type from a pop-up menu.

Sample usage (e.g. from within a wx.Frame OnSave handler):

    imageExtensions = [('TIFF', 'tif'),
                       ('Windows Bitmap', 'bmp'),
                       ('JPEG', 'jpg'),
                       ('PNG', 'png'),
                       ('PNM', 'pnm')
                       ]

    dlg = TypedSaveFileDialog(parent=self,
                              title='Save Image',
                              initialdir='$HOME',
                              typeinfo=imageExtensions)

    if dlg.ShowModal() == wx.ID_OK:
       print '  chosen file name: ', dlg.filename
       print '  chosen type extension: ', dlg.ext


Note that the chosen type extension is not always deducible from
the filename. If the user first chooses an explicit filetype from
the pop-up box, but then manually changes the resulting filename,
it can be possible for the dialog to have exit values of e.g.
dlg.filename == 'image.png', but dlg.ext == 'jpg'.

"""
import wx
import os


class TypedSaveFileDialog(wx.Dialog):

    def __init__(self, parent, title, initialdir, typeinfo):
        wx.Dialog.__init__(self, parent, -1, title)
        self.currentdir = os.path.abspath(os.path.expanduser(os.path.expandvars(str(initialdir))))
        self.typeinfo = typeinfo
        if len(typeinfo) > 1:
            self.typeinfo = [("By Extension", None)] + self.typeinfo
        self.str2ext = dict(typeinfo)

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

        # Other Event Listeneres
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def CreateControls(self):
        locdefault = self.currentdir
        if locdefault != "" and locdefault[-1] != os.sep:
            locdefault += os.sep

        self.locLabel = wx.StaticText(self, -1, "&Location:", size=wx.DLG_UNIT(self, -1, -1), style=wx.ALIGN_RIGHT)

        self.locField = wx.TextCtrl(self, -1, size=wx.DLG_UNIT(self, 150, -1), style=wx.TE_PROCESS_ENTER)
        self.locField.SetValue(locdefault)
        self.locField.SetInsertionPointEnd()
        self.locField.SetToolTip(wx.ToolTip("The filename to save to."))
        self.locField.Bind(wx.EVT_TEXT_ENTER, self.OnLocFieldEnter)
        self.locField.Bind(wx.EVT_TEXT, self.OnLocFieldChar)

        self.browseButton = wx.Button(self, -1, "Browse...")
        self.browseButton.SetToolTip(wx.ToolTip("Open a file selector dialog box."))
        self.browseButton.Bind(wx.EVT_BUTTON, self.OnBrowse)

        self.typeLabel = wx.StaticText(self, -1, "Determine File Type:", size=wx.DLG_UNIT(self, -1, -1),
                                       style=wx.ALIGN_RIGHT)

        self.typeChoice = wx.Choice(self, -1, choices=[desc for (desc, ext) in self.typeinfo])
        self.typeChoice.Enable(len(self.typeinfo) > 1)
        self.typeChoice.SetSelection(0)
        self.typeChoice.SetToolTip(wx.ToolTip("Available formats (this will auto-update filename if appropriate)."))
        self.typeChoice.Bind(wx.EVT_CHOICE, self.OnTypeChoice)

        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.cancelButton.Bind(wx.EVT_BUTTON, self.OnCancel)

        self.saveButton = wx.Button(self, wx.ID_OK, "Save")
        self.saveButton.Enable(False)
        self.saveButton.Bind(wx.EVT_BUTTON, self.OnSave)

        self.SetDefaultItem(self.locField)

        self.locField.SetFocus()

    def CreateLayout(self):
        hsizer1 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer1.Add(self.locField, 1, wx.EXPAND | wx.RIGHT, border=10)
        hsizer1.Add(self.browseButton, 0, wx.EXPAND)

        hsizer2 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer2.Add(self.typeLabel, 0, wx.ALIGN_CENTER | wx.RIGHT, border=10)
        hsizer2.Add(self.typeChoice, 0, wx.ALIGN_CENTER)

        hsizer3 = wx.BoxSizer(wx.HORIZONTAL)
        hsizer3.Add((0, 0), 1, wx.EXPAND)
        hsizer3.Add(self.cancelButton, 0, wx.EXPAND | wx.RIGHT, border=10)
        hsizer3.Add(self.saveButton, 0, wx.EXPAND)

        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(self.locLabel, 0, wx.LEFT | wx.TOP | wx.RIGHT, border=10)
        vsizer.Add(hsizer1, 0, wx.EXPAND | wx.ALL, border=10)
        vsizer.Add(hsizer2, 0, wx.EXPAND | wx.ALL, border=10)
        vsizer.Add(hsizer3, 0, wx.EXPAND | wx.LEFT | wx.BOTTOM | wx.RIGHT, border=10)
        vsizer.Fit(self)

        self.SetAutoLayout(True)
        self.SetSizerAndFit(vsizer)
        self.CentreOnParent()

    def OnTypeChoice(self, event):
        typestring = str(self.typeChoice.GetStringSelection())
        if typestring == "By Extension":
            return

        filename = str(self.locField.GetValue())
        if not filename:
            return

        base, ext = os.path.splitext(filename)
        if not base.endswith(os.sep):
            newfilename = base + "." + self.str2ext[typestring]
            self.locField.SetValue(newfilename)
        self.locField.SetInsertionPointEnd()

    def OnClose(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)

    def OnSave(self, event):
        self.filename = str(self.locField.GetValue())
        self.filetype = str(self.typeChoice.GetStringSelection())

        self.filename = os.path.abspath(os.path.expanduser(os.path.expandvars(self.filename)))

        if self.filetype != "By Extension":
            self.ext = self.str2ext[self.filetype]
        else:
            base, self.ext = os.path.splitext(self.filename)
            if not self.ext or self.ext == '.':
                wx.MessageBox(parent=self, message="Filename error: '%s' has no extension" % self.filename,
                              caption="Save Error", style=wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
                return
            self.ext = self.ext[1:]
            if self.ext not in self.str2ext.values():
                wx.MessageBox(parent=self, message="Filename error: Unknown extension '%s'" % self.ext,
                              caption="Save Error", style=wx.OK | wx.ICON_ERROR | wx.STAY_ON_TOP)
                return

        if os.path.exists(self.filename):
            overwrite = wx.MessageBox(parent=self,
                                      message="File '%s' already exists. Do you want to overwrite it?" % self.filename,
                                      caption="Confirm Overwrite",
                                      style=wx.OK | wx.CANCEL | wx.ICON_EXCLAMATION | wx.STAY_ON_TOP)

            if overwrite != wx.OK:
                return

        self.EndModal(wx.ID_OK)

    def OnLocFieldChar(self, event):
        self.saveButton.Enable(str(self.locField.GetValue()) != "")

    def OnLocFieldEnter(self, event):
        self.OnSave(None)

    def OnBrowse(self, event):
        directory = os.path.dirname(str(self.locField.GetValue()))
        if not directory:
            directory = self.currentdir
        directory = os.path.abspath(os.path.expanduser(os.path.expandvars(directory)))
        filename = wx.FileSelector("Save As...", default_path=directory, flags=wx.FD_SAVE, parent=self)
        if filename:
            self.locField.SetValue(filename)
            self.locField.SetInsertionPointEnd()
            self.saveButton.SetFocus()
            self.currentdir = os.path.dirname(filename)



# This is an example VISAN script for the GOME-2 O3MOTO product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOME-2 O3MOTO products (in HDF5 format).

# This example will then take all products it finds in this directory and
# for these products plot the total ozone columns


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'GOME_*_L2_*.HDF5'
    files = glob.glob(os.path.join(productdir, "GOME_*_L2_*.HDF5"))

    if len(files) == 0:
        print(("Could not find any GOME2 L2 files in directory '" + productdir + "'"))
        return

    # All backward scans are left out of the ingestion.
    product = harp.import_product(files, 'scan_direction_type=="forward"')

    wplot(product, value="O3_column_number_density", colorrange=(150, 500), numcolorlabels=8, colortable="Ozone",
          showpropertypanel=True)


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import numpy
import wx

from .worldplotwindow import WorldPlotWindow


class GeolocationPanel(wx.Panel):

    def __init__(self, parent):
        panelstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            panelstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=panelstyle)

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()
        self.datasetsShown = []

    def CreateControls(self):
        # Create the plot window
        self.plotWindow = WorldPlotWindow(self, -1)
        datadir = str(wx.Config.Get().Read('DirectoryLocation/ApplicationData'))
        self.plotWindow.SetCoastLineFile(os.path.join(datadir, "gshhs_l.b"))
        self.plotWindow.SetPoliticalBorderFile(os.path.join(datadir, "wdb_borders_l.b"))

    def CreateLayout(self):
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.plotWindow, 1, wx.EXPAND)
        self.SetSizer(sizer)

    def UpdateLocation(self, datasetId, location, keyframe):
        if len(location) == 0:
            self.plotWindow.SetViewCenter(0, 0)
            return
        latitude = location['latitude']
        longitude = location['longitude']
        if not isinstance(latitude, numpy.ndarray):
            try:
                latitude = numpy.asarray(latitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("location latitude cannot be converted to numpy array")
        if not isinstance(longitude, numpy.ndarray):
            try:
                longitude = numpy.asarray(longitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("location longitude cannot be converted to numpy array")
        if datasetId not in self.datasetsShown:
            if latitude.ndim == 2 and longitude.ndim == 2:
                self.plotWindow.AddSwathData(latitude, longitude, None)
            else:
                self.plotWindow.AddPointData(latitude.flat, longitude.flat, None)
            self.datasetsShown.append(datasetId)
        if latitude.ndim > 0 and longitude.ndim > 0:
            latitude = latitude[keyframe]
            longitude = longitude[keyframe]
        latitude = latitude.flat[0]
        longitude = longitude.flat[0]
        self.plotWindow.SetViewCenter(latitude, longitude)
        self.plotWindow.Refresh()



# This is an example VISAN script for GOMOS L2 products using CODA

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOM_NL__2P products.

# This example will then take the first product it finds in this directory and
# for that product plot the ozone profile


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all GOMOS L2 files in productdir.
    files = glob.glob(os.path.join(productdir, "GOM_NL__2P*"))
    if len(files) == 0:
        print(("Could not find any GOM_NL__2P files in directory '" + productdir + "'"))
        return

    # Ingest the ozone profile and tangent altitudes from the first GOMOS L2
    # product file.

    pf = coda.open(files[0])
    tangent_alt = coda.fetch(pf, 'nl_geolocation', -1, 'tangent_alt')
    o3 = coda.fetch(pf, 'nl_local_species_density', -1, 'o3')

    # convert altitude to km
    tangent_alt /= 1000

    # plot height vs. O3
    plot(o3, tangent_alt, title="GOMOS Level-2: Ozone profile", xlabel="Local O3 density at tangent height [ cm^-3 ]",
         ylabel="height\n[ km ]", yrange=(0, 75))

    coda.close(pf)


run()



# This is an example VISAN script for the SCI_OL__2P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing SCI_OL__2P products.

# This example will then take all products it finds in this directory and
# for these products plot the total ozone columns from the nadir mode


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'SCI_OL__2P'
    files = glob.glob(os.path.join(productdir, "SCI_OL__2P*"))

    if len(files) == 0:
        print(("Could not find any SCI_OL__2P files in directory '" + productdir + "'"))
        return

    # We ingest the ozone data that was retrieved using UV/VIS fitting window no. 0.
    # All backward scans are left out of the ingestion.
    # Finally we have the ingestion routine convert the VCD values to Dobson Units.
    data = harp.import_product(files, 'scan_direction_type!="backward";derive(O3_column_number_density [DU])',
                               'dataset=nad_uv0_o3')

    wplot(data, value="O3_column_number_density", colorrange=(150, 500), numcolorlabels=8, colortable="Ozone",
          showpropertypanel=True)


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import wx
import wx.lib.newevent

from .util import DetermineCharSize

KeyframeEvent, EVT_KEYFRAME_CHANGED = wx.lib.newevent.NewEvent()


class AnimationToolbar(wx.Panel):

    """ This class can be used to create animation. It fires an
    EVT_KEYFRAME_CHANGED if the slider has moved either by hand
    or by the integrated Timer control"""

    def __init__(self, parent, plotWindow):
        panelStyle = wx.TAB_TRAVERSAL | wx.SUNKEN_BORDER
        if sys.platform == 'win32':
            # We need to add the full repaint on resize property on Windows to force proper drawing of the controls
            panelStyle = panelStyle | wx.FULL_REPAINT_ON_RESIZE

        wx.Panel.__init__(self, parent, -1, style=panelStyle)

        # Initialize state
        self.plotWindow = plotWindow
        self.currentKeyframe = 0
        self.numKeyframes = plotWindow.GetNumKeyframes()
        self.interval = 50
        self.running = False
        self.loop = False

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

    def CreateControls(self):
        self.playString = "Play"
        self.pauseString = "Pause"
        # NOTE: The following lines would cause a segmentation fault. I suspect
        # that internally, ToolTips (or their C proxies) are deleted
        # regardless of Python refcount, so you can't reuse them like
        # this (similar to what is documented for Icons).
        #
        # self.playtip  = wx.ToolTip("Display the frames in the plot in sequence")
        # self.pausetip = wx.ToolTip("Pause the plot frame animation")
        self.playtipstr = "Start running through the frames in the plot in sequence."
        self.pausetipstr = "Pause the plot frame animation."

        (charWidth, charHeight) = DetermineCharSize(self)

        maxRangeValue = self.numKeyframes - 1
        if self.numKeyframes <= 1:
            # GTK2 doesn't like it if 'min' equals 'max' for a range
            maxRangeValue = 1

        self.TIMER = wx.Timer(self, -1)
        self.Bind(wx.EVT_TIMER, self.OnTimer)

        self.PLAY = wx.Button(self, -1, self.playString)
        self.PLAY.SetToolTip(wx.ToolTip(self.playtipstr))
        self.PLAY.Bind(wx.EVT_BUTTON, self.OnAnimate)

        self.SLIDER = wx.Slider(self, -1, value=0, minValue=0, maxValue=maxRangeValue)
        self.SLIDER.SetToolTip(wx.ToolTip("Pause the plot frame animation."))
        self.SLIDER.Bind(wx.EVT_COMMAND_SCROLL, self.OnScroll)

        self.RESET = wx.Button(self, -1, "Reset")
        self.RESET.SetToolTip(wx.ToolTip("Drag this slider to manually animate the plot frames."))
        self.RESET.Bind(wx.EVT_BUTTON, self.OnReset)

        self.KEYFRAME = wx.SpinCtrl(self, -1, str(0), size=(5 * charWidth, -1),
                                    style=wx.SP_WRAP | wx.SP_ARROW_KEYS | wx.TE_PROCESS_ENTER | wx.TE_RIGHT,
                                    min=0, max=maxRangeValue, initial=0)
        self.KEYFRAME.SetToolTip(wx.ToolTip("Enter the number of a plot frame to display. "
                                            "Use the arrows to increment/decrement by one."))
        self.KEYFRAME.Bind(wx.EVT_SPINCTRL, self.OnKeyframeSpin)
        self.KEYFRAME.Bind(wx.EVT_TEXT_ENTER, self.OnKeyframeText)

        self.LOOP = wx.CheckBox(self, -1, "Loop")
        self.LOOP.SetToolTip(wx.ToolTip("Automatically restart the animation at the first plot frame."))
        self.LOOP.SetValue(self.loop)
        self.LOOP.Bind(wx.EVT_CHECKBOX, self.OnLoop)

        self.SPEEDLABEL = wx.StaticText(self, -1, "Speed:")
        self.SPEEDLABEL.SetToolTip(wx.ToolTip("Enter the animation speed to use, in frames per second. "
                                              "Use the arrows to increment/decrement by one."))

        self.SPEEDFIELD = wx.SpinCtrl(self, -1, str(self.GetFPS()), size=(5 * charWidth, -1),
                                      style=wx.SP_ARROW_KEYS | wx.TE_PROCESS_ENTER, min=1, max=60,
                                      initial=self.GetFPS())
        self.SPEEDFIELD.SetToolTip(wx.ToolTip("Enter the animation speed to use, in frames per second. "
                                              "Use the arrows to increment/decrement by one."))
        self.SPEEDFIELD.Bind(wx.EVT_SPINCTRL, self.OnSpeedSpin)
        self.SPEEDFIELD.Bind(wx.EVT_TEXT_ENTER, self.OnSpeedText)

    def CreateLayout(self):
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(self.PLAY, 0, wx.ALIGN_CENTER | wx.ALL, border=5)
        hsizer.Add(self.SLIDER, 1, wx.EXPAND | wx.RIGHT, border=5)
        hsizer.Add(self.RESET, 0, wx.ALIGN_CENTER | wx.RIGHT, border=5)
        hsizer.Add(self.KEYFRAME, 0, wx.ALIGN_CENTER | wx.RIGHT, border=5)
        hsizer.Add(wx.StaticLine(self, -1, style=wx.LI_VERTICAL), 0, wx.EXPAND | wx.RIGHT, border=5)
        hsizer.Add(self.LOOP, 0, wx.ALIGN_CENTER | wx.RIGHT, border=5)
        hsizer.Add(wx.StaticLine(self, -1, style=wx.LI_VERTICAL), 0, wx.EXPAND | wx.RIGHT, border=5)
        hsizer.Add(self.SPEEDLABEL, 0, wx.ALIGN_CENTER | wx.RIGHT, border=5)

        # On MacOS, the right lower corner of a frame is taken up by the resize window widget, so we must keep our
        # animation toolbar controls away from it. Since gratuituous spacing is butt-ugly on other platforms, I
        # am solving this by special-casing for the Mac.
        if '__WXMAC__' in wx.PlatformInfo:
            endspace = 15
        else:
            endspace = 5

        hsizer.Add(self.SPEEDFIELD, 0, wx.ALIGN_CENTER | wx.RIGHT, border=endspace)

        self.SetSizerAndFit(hsizer)

        self.PLAY.SetFocus()

    def UpdateNumKeyframes(self):
        self.numKeyframes = self.plotWindow.GetNumKeyframes()

        hascurrentfocus = wx.Window.FindFocus()
        maxRangeValue = self.numKeyframes - 1
        if self.numKeyframes <= 1:
            # GTK2 doesn't like it if 'min' equals 'max' for a range
            maxRangeValue = 1
        self.SLIDER.SetRange(0, maxRangeValue)
        self.KEYFRAME.SetRange(0, maxRangeValue)
        if hascurrentfocus:
            hascurrentfocus.SetFocus()

    def SetFPS(self, fps):
        """ Set Timer interval in FPS. Restarts timer if it is currently running """
        self.interval = int((1.0 / float(fps)) * 1000.0)
        self.SPEEDFIELD.SetValue(fps)
        if (self.running):
            self.TIMER.Start(self.interval)

    def GetFPS(self):
        """ Return Timer interval in FPS """
        return int((1.0 / self.interval) * 1000.0)

    def SetIntervalMSec(self, ms):
        """ Set interval to ms (msec) """
        self.interval = ms
        if (self.running):
            self.TIMER.Start(self.interval)

    def SetLoop(self, loop):
        """ Set whether the slider should loop on end """
        self.LOOP.SetValue(loop)
        self.loop = loop

    def GetLoop(self):
        """ Get the loop state """
        return self.loop

    def SetKeyframe(self, keyframe):
        """ Set the current keyframe """
        keyframe = min(keyframe, self.numKeyframes - 1)
        if keyframe != self.currentKeyframe:
            self.currentKeyframe = keyframe
            self.SLIDER.SetValue(keyframe)
            self.KEYFRAME.SetValue(keyframe)
            self.Update()
            wx.PostEvent(self, KeyframeEvent(keyframe=keyframe))

    def GetKeyframe(self):
        """ Get the current keyframe """
        return self.currentKeyframe

    def Play(self):
        """ Start animation """
        if self.currentKeyframe == self.numKeyframes - 1:
            self.Reset()

        self.running = True
        self.TIMER.Start(self.interval)
        self.PLAY.SetLabel(self.pauseString)
        self.PLAY.SetToolTip(wx.ToolTip(self.pausetipstr))
        self.KEYFRAME.Enable(False)

    def Pause(self):
        """ Pause animation """
        self.running = False
        self.TIMER.Stop()
        self.PLAY.SetLabel(self.playString)
        self.PLAY.SetToolTip(wx.ToolTip(self.playtipstr))
        self.KEYFRAME.Enable(True)

    def Reset(self):
        """ Reset animation """
        self.SetKeyframe(0)

    def TogglePlayPause(self):
        """ Toggle animation Play / Pause """
        if self.running:
            self.Pause()
        else:
            self.Play()

    def Destroy(self):
        self.Pause()
        wx.Panel.Destroy(self)

    def OnLoop(self, event):
        self.SetLoop(event.IsChecked())

    def OnSpeedSpin(self, event):
        self.SetFPS(event.GetPosition())

    def OnSpeedText(self, event):
        self.SetFPS(int(event.GetString()))

    def OnScroll(self, event):
        """ Event Handler for Slider-Scroll """
        self.SetKeyframe(self.SLIDER.GetValue())

    def OnAnimate(self, event):
        self.TogglePlayPause()

    def OnStart(self, event):
        self.Play()

    def OnPause(self, event):
        self.Pause()

    def OnReset(self, event):
        """ Event Handler Reset Button """
        self.Reset()

    def OnTimer(self, event):
        """ Event Handler for Timer Event """
        # Discard timer events still arriving after animation has been stopped.
        if not self.running:
            return

        if self.currentKeyframe < self.numKeyframes - 1:
            # Advance 1
            self.SetKeyframe(self.currentKeyframe + 1)
        else:
            # Auto stop
            if self.loop:
                self.Reset()
            else:
                self.Pause()

    def OnKeyframeSpin(self, event):
        self.SetKeyframe(event.GetPosition())

    def OnKeyframeText(self, event):
        self.SetKeyframe(int(event.GetString()))



# This is an example VISAN script for the GOM_NL__2P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOM_NL__2P products.

# This example will then plot the ozone profiles for all GOMOS L2 products in that directory


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'GOM_NL__2P'
    files = glob.glob(os.path.join(productdir, "GOM_NL__2P*"))

    if len(files) == 0:
        print(("Could not find any GOM_NL__2P files in directory '" + productdir + "'"))
        return

    # We ingest the ozone profile data
    product = harp.import_product(files)

    plot(product, xlabel="value [cm-3]", showpropertypanel=True, name="o3 profile",
         title="GOM_NL__2P profile example (o3)")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os

import wx
import numpy

from .animationtoolbar import AnimationToolbar, EVT_KEYFRAME_CHANGED
from .typedsavefiledialog import TypedSaveFileDialog
from .worldplotdatasetpanel import WorldPlotDataSetPanel, EVT_CURRENTDATASET_CHANGED
from .worldplotpropertypanel import WorldPlotPropertyPanel
from .worldplotwindow import WorldPlotWindow, EVT_WORLDPLOTDATA_CHANGED, EVT_WORLDVIEW_CHANGED, PROJECTIONS

windowCount = 1


class WorldPlotFrame(wx.Frame):

    def __init__(self, parent=None, id=-1, title=None, pos=wx.DefaultPosition, size=(800, 494)):
        if parent is None:
            # use top level wxWindow
            try:
                parent = wx.GetApp().GetTopWindow()
            except Exception:
                # If there is no top level window, just create parentless plot window
                pass
        if title is None:
            # determine title
            global windowCount
            title = "VISAN World Plot"
            if windowCount > 1:
                title += " - %d" % windowCount
            windowCount += 1
        wx.Frame.__init__(self, parent, id, title, pos, size)

        # Set icon if possible
        if wx.Config.Get().Read("IconFile"):
            self.SetIcon(wx.Icon(wx.Config.Get().Read("IconFile")))

        # Set initial state
        self.closingDown = False
        self.filename = ""
        self.dataSetAttributes = []

        # Create and configure all widgets
        self.CreateMenuBar()
        self.CreateControls()
        self.CreateLayout()

        # Other Event Listeneres
        self.Bind(wx.EVT_CLOSE, self.OnClose)

        # Give the plotwindow the initial focus (since this might not happen automatically for e.g. Mac OS X)
        self.plotWindow.SetFocus()

        self.Show()
        wx.Yield()

    def __repr__(self):
        return "<visan worldplot handle>"

    def CreateMenuBar(self):
        menubar = wx.MenuBar()

        filemenu = wx.Menu()

        item = filemenu.Append(wx.ID_SAVE, "&Save Image...\tCtrl-S", "Save the image to an image file")
        self.Bind(wx.EVT_MENU, self.OnSave, item)

        filemenu.AppendSeparator()

        item = filemenu.Append(wx.ID_CLOSE, "&Close\tCtrl-W", "Close this Window")
        self.Bind(wx.EVT_MENU, self.OnClose, item)

        menubar.Append(filemenu, "&File")

        viewmenu = wx.Menu()

        self.viewPropertiesMenuItem = viewmenu.AppendCheckItem(wx.ID_ANY, "&Properties",
                                                               "Toggle the display of the Property Panel")
        self.viewPropertiesMenuItem.Enable(True)
        self.Bind(wx.EVT_MENU, self.OnViewProps, self.viewPropertiesMenuItem)

        viewmenu.AppendSeparator()

        self.viewColorBarMenuItem = viewmenu.AppendCheckItem(wx.ID_ANY, "&Color Bar",
                                                             "Toggle the display of the Color Bar")
        self.Bind(wx.EVT_MENU, self.OnViewColorBar, self.viewColorBarMenuItem)

        self.viewSliderMenuItem = viewmenu.AppendCheckItem(wx.ID_ANY, "&Animation Toolbar",
                                                           "Toggle the display of the Animation Toolbar")
        self.viewSliderMenuItem.Enable(False)
        self.Bind(wx.EVT_MENU, self.OnViewSlider, self.viewSliderMenuItem)

        menubar.Append(viewmenu, "&View")

        self.SetMenuBar(menubar)

    def CreateControls(self):
        # Create a split panel
        splitterstyle = wx.SP_LIVE_UPDATE
        if wx.Platform == "__WXMAC__":
            splitterstyle |= wx.SP_3DSASH
        self.splitPanel = wx.SplitterWindow(self, -1, style=splitterstyle)

        # Create the plot window
        self.plotWindow = WorldPlotWindow(self.splitPanel, -1)
        datadir = str(wx.Config.Get().Read('DirectoryLocation/ApplicationData'))
        self.plotWindow.SetCoastLineFile(os.path.join(datadir, "gshhs_l.b"))
        self.plotWindow.SetPoliticalBorderFile(os.path.join(datadir, "wdb_borders_l.b"))
        self.plotWindow.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.plotWindow.Bind(EVT_WORLDPLOTDATA_CHANGED, self.OnPlotDataChanged)
        self.plotWindow.Bind(EVT_WORLDVIEW_CHANGED, self.OnWorldViewChanged)

        # Create the animation toolbar
        self.animationToolbar = AnimationToolbar(self, self.plotWindow)
        self.animationToolbar.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.animationToolbar.Bind(EVT_KEYFRAME_CHANGED, self.OnKeyframeChanged)

        # Create the property panel
        # Create a parent wxWindow for the notebook to allow for a background colour for the area behind the tabs
        self.propertyPanel = wx.Window(self.splitPanel, -1)
        # Set the background 'explicitly' to have it stick; TODO: still needed?
        self.propertyPanel.SetBackgroundColour(self.propertyPanel.GetBackgroundColour())
        self.propertyPanel.Bind(wx.EVT_SIZE, self.OnPropertyPanelSize)

        # Create the property notebook
        self.propertyNotebook = wx.Notebook(self.propertyPanel, -1)
        # Don't add size handler for propertyNotebook: re-layout of propertyNotebook doesn't work on Windows

        # Create the content for the property notebook
        self.dataSetPropertyTab = WorldPlotDataSetPanel(self.propertyNotebook, self, self.plotWindow)
        self.dataSetPropertyTab.Bind(wx.EVT_SIZE, self.OnDataSetPropertyTabSize)
        self.dataSetPropertyTab.Bind(EVT_CURRENTDATASET_CHANGED, self.OnCurrentDataSetChanged)
        self.plotPropertyTab = WorldPlotPropertyPanel(self.propertyNotebook, self.plotWindow)
        self.plotPropertyTab.Bind(wx.EVT_SIZE, self.OnPlotPropertyTabSize)

        # Add the content to the notebook
        self.propertyNotebook.AddPage(self.dataSetPropertyTab, "Datasets")
        self.propertyNotebook.AddPage(self.plotPropertyTab, "Plot")

        # Initialize split panel
        self.splitPanel.SetMinimumPaneSize(50)
        self.splitPanel.Initialize(self.plotWindow)
        self.splitPanel.Bind(wx.EVT_SIZE, self.OnSplitPanelSize)

    def CreateLayout(self):
        self.propertyPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.propertyPanelSizer.Add(self.propertyNotebook, 1, wx.EXPAND)
        self.propertyPanel.SetSizerAndFit(self.propertyPanelSizer)

        self.splitPanelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.splitPanelSizer.Add(self.plotWindow, 1, wx.EXPAND)
        self.splitPanelSizer.Add((self.splitPanel.GetSashSize(), -1), 0, wx.EXPAND)
        self.splitPanelSizer.Add(self.propertyPanel, 0, wx.EXPAND)
        self.splitPanelSizer.Show(1, False)
        self.splitPanelSizer.Show(2, False)
        self.splitPanel.SetSizer(self.splitPanelSizer)

        self.plotWindow.SetMinSize((80, 50))

        self.verticalSizer = wx.BoxSizer(wx.VERTICAL)
        self.verticalSizer.Add(self.splitPanel, 1, wx.EXPAND)
        self.verticalSizer.Add(self.animationToolbar, 0, wx.EXPAND)

        # Hide the animation toolbar by default (so it must be shown explicitly)
        self.verticalSizer.Hide(self.animationToolbar)

        # Hide the property panel by default (so it must be shown explicitly)
        self.propertyPanel.Hide()

        self.SetSizer(self.verticalSizer)
        self.Layout()
        self.UpdateMinSize()

    def UpdateMinSize(self):
        self.SetMinSize(self.GetBestSize())
        currentsize = self.GetSize()
        minsize = self.GetMinSize()
        if currentsize.x < minsize.x or currentsize.y < minsize.y:
            if currentsize.x < minsize.x:
                currentsize.x = minsize.x
            if currentsize.y < minsize.y:
                currentsize.y = minsize.y
            self.SetSize(currentsize)

    def AdjustSplitPanelSashPosition(self):
        # Update sash position in splitter window
        # Try to keep the size of the property pane the same
        sashPosition = -(self.propertyPanel.GetSize()[0] + self.splitPanel.GetSashSize())
        self.splitPanel.SetSashPosition(sashPosition)

    def ShowAnimationToolbar(self, show=True):
        try:
            show = bool(show)
        except ValueError:
            raise TypeError("Show Animation Toolbar parameter should be a boolean (was: '%s')" % str(show))
        self.viewSliderMenuItem.Check(show)
        if show:
            self.viewSliderMenuItem.Enable()

        self.verticalSizer.Show(self.animationToolbar, show)
        self.Layout()

        self.UpdateMinSize()
        self.Refresh()

    def ShowPropertyPanel(self, show=True):
        try:
            show = bool(show)
        except ValueError:
            raise TypeError("Show Property Panel parameter should be a boolean (was: '%s')" % str(show))
        self.viewPropertiesMenuItem.Check(show)

        if show:
            self.splitPanel.SplitVertically(self.plotWindow, self.propertyPanel)
            self.splitPanelSizer.Show(1, True)
            self.splitPanelSizer.Show(2, True)
            self.propertyPanel.Show()
            self.propertyPanel.Fit()
            self.AdjustSplitPanelSashPosition()
        else:
            self.splitPanel.Unsplit(self.propertyPanel)
            self.splitPanelSizer.Show(1, False)
            self.splitPanelSizer.Show(2, False)
        self.splitPanel.Layout()

        self.UpdateMinSize()
        self.Refresh()

    def ShowColorBar(self, show=True):
        try:
            show = bool(show)
        except ValueError:
            raise TypeError("Show Color Bar parameter should be a boolean (was: '%s')" % str(show))
        self.viewColorBarMenuItem.Check(show)
        self.plotWindow.ShowColorBar(show)

    def SelectDataSet(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        self.dataSetPropertyTab.SelectDataSet(dataSetId)
        self.plotWindow.SetDataSetForColorBar(dataSetId)
        self.plotPropertyTab.UpdateControls()

    def GetSelectedDataSet(self):
        return self.plotWindow.GetDataSetForColorBar()

    def AddPointData(self, latitude, longitude, data=None):
        # convert data to numpy array if available
        dataArray = None
        if data is not None:
            if isinstance(data, numpy.ndarray):
                dataArray = data
            else:
                try:
                    dataArray = numpy.asarray(data, dtype=numpy.double)
                except TypeError:
                    raise TypeError("plot data argument cannot be converted to numpy array. %s" % str(data))
            if dataArray.ndim == 0:
                # convert scalar to 1D array with one element
                dataArray.setshape([1])
            if dataArray.ndim > 2:
                raise ValueError("Cannot plot a %d-dimensional numpy array" % dataArray.ndim)

        # convert latitude to numpy latitudeArray
        if isinstance(latitude, numpy.ndarray):
            latitudeArray = latitude
        else:
            try:
                latitudeArray = numpy.asarray(latitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot latitude argument cannot be converted to numpy array. %s" % str(latitude))
        if latitudeArray.ndim == 0:
            # convert scalar to 1D array with one element
            latitudeArray = numpy.resize(latitudeArray, [1])
        if latitudeArray.ndim > 2:
            raise ValueError("plot latitude argument cannot be a %d-dimensional numpy array" % latitudeArray.ndim)
        if data is not None and len(data) != len(latitudeArray):
            raise ValueError("plot latitude and data arguments do not have the same size")

        # convert longitude to numpy longitudeArray
        if isinstance(longitude, numpy.ndarray):
            longitudeArray = longitude
        else:
            try:
                longitudeArray = numpy.asarray(longitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot longitude argument cannot be converted to numpy array. %s" % str(longitude))
        if longitudeArray.ndim == 0:
            # convert scalar to 1D array with one element
            longitudeArray = numpy.resize(longitudeArray, [1])
        if longitudeArray.ndim > 2:
            raise ValueError("plot longitude argument cannot be a %d-dimensional numpy array" % longitudeArray.ndim)
        if len(latitudeArray) != len(longitudeArray):
            raise ValueError("plot latitude and longitude arguments do not have the same size")

        # create empty attribute entry for this data set
        self.dataSetAttributes.extend([dict()])

        dataSetId = self.plotWindow.AddPointData(latitudeArray, longitudeArray, dataArray)

        self.plotPropertyTab.UpdateControls()

        return dataSetId

    def AddLineData(self, latitude, longitude):
        # convert latitude to numpy latitudeArray
        if isinstance(latitude, numpy.ndarray):
            latitudeArray = latitude
        else:
            try:
                latitudeArray = numpy.asarray(latitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot latitude argument cannot be converted to numpy array. %s" % str(latitude))
        if latitudeArray.ndim == 0:
            # convert scalar to 1D array with one element
            latitudeArray = numpy.resize(latitudeArray, [1])
        if latitudeArray.ndim > 2:
            raise ValueError("plot latitude argument cannot be a %d-dimensional numpy array" % latitudeArray.ndim)

        # convert longitude to numpy longitudeArray
        if isinstance(longitude, numpy.ndarray):
            longitudeArray = longitude
        else:
            try:
                longitudeArray = numpy.asarray(longitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot longitude argument cannot be converted to numpy array. %s" % str(longitude))
        if longitudeArray.ndim == 0:
            # convert scalar to 1D array with one element
            longitudeArray = numpy.resize(longitudeArray, [1])
        if longitudeArray.ndim > 2:
            raise ValueError("plot longitude argument cannot be a %d-dimensional numpy array" % longitudeArray.ndim)
        if len(latitudeArray) != len(longitudeArray):
            raise ValueError("plot latitude and longitude arguments do not have the same size")

        # create empty attribute entry for this data set
        self.dataSetAttributes.extend([dict()])

        dataSetId = self.plotWindow.AddLineData(latitudeArray, longitudeArray)

        self.plotPropertyTab.UpdateControls()

        return dataSetId

    def AddSwathData(self, cornerLatitude, cornerLongitude, data=None):
        # convert data to numpy array if available
        dataArray = None
        if data is not None:
            if isinstance(data, numpy.ndarray):
                dataArray = data
            else:
                try:
                    dataArray = numpy.asarray(data, dtype=numpy.double)
                except TypeError:
                    raise TypeError("plot data argument cannot be converted to numpy array. %s" % str(data))
            if dataArray.ndim == 0:
                # convert scalar to 1D array with one element
                dataArray.setshape([1])
            if dataArray.ndim > 2:
                raise ValueError("plot data argument cannot be a %d-dimensional numpy array" % dataArray.ndim)

        # convert latitude to numpy latitudeArray
        if isinstance(cornerLatitude, numpy.ndarray):
            latitudeArray = cornerLatitude
        else:
            try:
                latitudeArray = numpy.asarray(cornerLatitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot latitude argument cannot be converted to numpy array. %s" % str(cornerLatitude))
        if latitudeArray.ndim == 1:
            # convert single swath to swath array
            latitudeArray = numpy.resize(latitudeArray, [1, len(latitudeArray)])
        if latitudeArray.ndim != 2:
            raise ValueError("plot cornerLatitude argument cannot be a %d-dimensional numpy array" % latitudeArray.ndim)
        if latitudeArray.shape[1] != 2 and latitudeArray.shape[1] != 4:
            raise ValueError("last dimension of plot cornerLatitude argument should be 2 or 4 but not %d" %
                             latitudeArray.shape[1])
        if data is not None and len(data) != len(latitudeArray):
            raise ValueError("plot cornerLatitude and data arguments do not have the same number of elements")

        # convert longitude to numpy longitudeArray
        if isinstance(cornerLongitude, numpy.ndarray):
            longitudeArray = cornerLongitude
        else:
            try:
                longitudeArray = numpy.asarray(cornerLongitude, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot longitude argument cannot be converted to numpy array. %s" % str(cornerLongitude))
        if longitudeArray.ndim == 1:
            # convert single swath to swath array
            longitudeArray = numpy.resize(longitudeArray, [1, len(longitudeArray)])
        if longitudeArray.ndim != 2:
            raise ValueError("plot cornerLongitude argument cannot be a %d-dimensional numpy array" %
                             longitudeArray.ndim)
        if longitudeArray.shape[1] != 2 and longitudeArray.shape[1] != 4:
            raise ValueError("last dimension of plot cornerLongitude argument should be 2 or 4 but not %d" %
                             longitudeArray.shape[1])
        if latitudeArray.shape != longitudeArray.shape:
            raise ValueError("plot cornerLatitude and cornerLongitude arguments do not have the same shape")

        if latitudeArray.shape[1] == 2:
            # convert bounding rect coordinates to bounding polygons
            newLat = numpy.zeros(shape=(latitudeArray.shape[0], 4))
            newLon = numpy.zeros(shape=newLat.shape)
            newLat[:, 0] = latitudeArray[:, 0]
            newLon[:, 0] = longitudeArray[:, 0]
            newLat[:, 1] = latitudeArray[:, 0]
            newLon[:, 1] = longitudeArray[:, 1]
            newLat[:, 2] = latitudeArray[:, 1]
            newLon[:, 2] = longitudeArray[:, 1]
            newLat[:, 3] = latitudeArray[:, 1]
            newLon[:, 3] = longitudeArray[:, 0]
            latitudeArray = newLat
            longitudeArray = newLon

        # create empty attribute entry for this data set
        self.dataSetAttributes.extend([dict()])

        dataSetId = self.plotWindow.AddSwathData(latitudeArray, longitudeArray, dataArray)

        if data is not None:
            self.ShowColorBar()

        self.plotPropertyTab.UpdateControls()

        return dataSetId

    # data should have size: [numGrids, numLatitudes, numLongitudes] or [numLatitudes, numLongitudes]
    def AddGridData(self, latitudeGrid, longitudeGrid, data):
        # convert data to numpy array
        if isinstance(data, numpy.ndarray):
            dataArray = data
        else:
            try:
                dataArray = numpy.asarray(data, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot data argument cannot be converted to numpy array. %s" % str(data))
        if dataArray.ndim == 2:
            # convert single grid to array with one grid element
            dataArray = numpy.resize(dataArray, [1, dataArray.shape[0], dataArray.shape[1]])
        if dataArray.ndim != 3:
            raise ValueError("plot data argument cannot be a %d-dimensional numpy array" % dataArray.ndim)

        # convert latitudeGrid to numpy latitudeArray
        if isinstance(latitudeGrid, numpy.ndarray):
            latitudeArray = latitudeGrid
        else:
            try:
                latitudeArray = numpy.asarray(latitudeGrid, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot latitudeGrid argument cannot be converted to numpy array. %s" % str(latitudeGrid))
        if latitudeArray.ndim != 1:
            if latitudeArray.ndim == 2:
                if latitudeArray.shape[0] != dataArray.shape[0]:
                    raise ValueError("plot latitudeGrid time dimension does not match time dimension of grid data")
                if not numpy.all(latitudeArray[:] == latitudeArray[0]):
                    raise ValueError("plot latitudeGrid should be the same for all grids")
                latitudeArray = latitudeArray[0]
            else:
                raise ValueError("plot latitudeGrid argument should be a one dimensional array")
        if latitudeArray.shape[0] != dataArray.shape[1]:
            raise ValueError("plot latitudeGrid size does not match latitude dimension of grid data")

        # convert longitudeGrid to numpy longitudeArray
        if isinstance(longitudeGrid, numpy.ndarray):
            longitudeArray = longitudeGrid
        else:
            try:
                longitudeArray = numpy.asarray(longitudeGrid, dtype=numpy.double)
            except TypeError:
                raise TypeError("plot longitudeGrid argument cannot be converted to numpy array. %s" %
                                str(longitudeGrid))
        if longitudeArray.ndim != 1:
            if longitudeArray.ndim == 2:
                if longitudeArray.shape[0] != dataArray.shape[0]:
                    raise ValueError("plot longitudeGrid time dimension does not match time dimension of grid data")
                if not numpy.all(longitudeArray[:] == longitudeArray[0]):
                    raise ValueError("plot longitudeGrid should be the same for all grids")
                longitudeArray = longitudeArray[0]
            else:
                raise ValueError("plot longitudeGrid argument should be a one dimensional array")
        if longitudeArray.shape[0] != dataArray.shape[2]:
            raise ValueError("plot longitudeGrid size does not match longitude dimension of grid data")

        # create empty attribute entry for this data set
        self.dataSetAttributes.extend([dict()])

        dataSetId = self.plotWindow.AddGridData(latitudeArray, longitudeArray, dataArray[0])
        if dataArray.shape[0] > 1:
            for i in range(1, dataArray.shape[0]):
                self.plotWindow.AddGridData(latitudeArray, longitudeArray, dataArray[i], dataSetId=dataSetId)
            self.ShowAnimationToolbar()

        self.ShowColorBar()
        self.plotPropertyTab.UpdateControls()

        return dataSetId

    def AddDataSetAttribute(self, dataSetId, name, value):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        self.dataSetAttributes[dataSetId][name] = value
        self.dataSetPropertyTab.UpdateAttributes()

    def GetNumDataSets(self):
        return self.plotWindow.GetNumDataSets()

    def GetPlotTitle(self):
        return self.plotWindow.GetPlotTitle()

    def SetPlotTitle(self, title):
        self.plotWindow.SetPlotTitle(str(title))
        self.plotPropertyTab.UpdateControls()

    def GetProjection(self):
        return self.plotWindow.GetProjection()

    def SetProjection(self, projection):
        if projection not in PROJECTIONS:
            raise ValueError("projection '%s' not supported (possible values are %s)" %
                             (projection, str(PROJECTIONS)))
        self.plotWindow.SetProjection(projection)
        self.plotPropertyTab.UpdateControls()

    def GetProjectionCenterLatitude(self):
        return self.plotWindow.GetProjectionCenterLatitude()

    def SetProjectionCenterLatitude(self, centerLatitude):
        try:
            centerLatitude = float(centerLatitude)
        except ValueError:
            raise TypeError("projCenterLatitude property should be a float (was: '%s')" % str(centerLatitude))

        if (centerLatitude < -90) or (centerLatitude > 90):
            raise ValueError("projCenterLatitude property should be between -90 and 90 (was: '%s')" %
                             str(centerLatitude))

        self.plotWindow.SetProjectionCenterLatitude(centerLatitude)
        self.plotPropertyTab.UpdateControls()

    def GetProjectionCenterLongitude(self):
        return self.plotWindow.GetProjectionCenterLongitude()

    def SetProjectionCenterLongitude(self, centerLongitude):
        try:
            centerLongitude = float(centerLongitude)
        except ValueError:
            raise TypeError("projCenterLongitude property should be a float (was: '%s')" % str(centerLongitude))

        # normalize to [-180,180]
        while centerLongitude < -180:
            centerLongitude += 360
        while centerLongitude > 180:
            centerLongitude -= 360

        self.plotWindow.SetProjectionCenterLongitude(centerLongitude)
        self.plotPropertyTab.UpdateControls()

    def GetViewCenterLatitude(self):
        return self.plotWindow.GetViewCenterLatitude()

    def SetViewCenterLatitude(self, centerLatitude):
        try:
            centerLatitude = float(centerLatitude)
        except ValueError:
            raise TypeError("viewCenterLatitude property should be a float (was: '%s')" % str(centerLatitude))

        if (centerLatitude < -90) or (centerLatitude > 90):
            raise ValueError("viewCenterLatitude property should be between -90 and 90 (was: '%s')" %
                             str(centerLatitude))

        centerLongitude = self.plotWindow.GetViewCenterLongitude()
        self.plotWindow.SetViewCenter(centerLatitude, centerLongitude)
        self.plotPropertyTab.UpdateControls()

    def GetViewCenterLongitude(self):
        return self.plotWindow.GetViewCenterLongitude()

    def SetViewCenterLongitude(self, centerLongitude):
        try:
            centerLongitude = float(centerLongitude)
        except ValueError:
            raise TypeError("viewCenterLongitude property should be a float (was: '%s')" % str(centerLongitude))

        # normalize to [-180,180]
        while centerLongitude < -180:
            centerLongitude += 360
        while centerLongitude > 180:
            centerLongitude -= 360

        centerLatitude = self.plotWindow.GetViewCenterLatitude()
        self.plotWindow.SetViewCenter(centerLatitude, centerLongitude)
        self.plotPropertyTab.UpdateControls()

    def SetViewCenter(self, centerLatitude, centerLongitude):
        try:
            centerLatitude = float(centerLatitude)
        except ValueError:
            raise TypeError("viewCenterLatitude property should be a float (was: '%s')" % str(centerLatitude))

        if (centerLatitude < -90) or (centerLatitude > 90):
            raise ValueError("viewCenterLatitude property should be between -90 and 90 (was: '%s')" %
                             str(centerLatitude))

        try:
            centerLongitude = float(centerLongitude)
        except ValueError:
            raise TypeError("viewCenterLongitude property should be a float (was: '%s')" % str(centerLongitude))

        # normalize to [-180,180]
        while centerLongitude < -180:
            centerLongitude += 360
        while centerLongitude > 180:
            centerLongitude -= 360

        self.plotWindow.SetViewCenter(centerLatitude, centerLongitude)
        self.plotPropertyTab.UpdateControls()

    def GetViewZoom(self):
        return self.plotWindow.GetViewZoom()

    def SetViewZoom(self, zoomScale):
        try:
            zoomScale = float(zoomScale)
        except ValueError:
            raise TypeError("zoomScale property should be a float (was: '%s')" % str(zoomScale))

        # clamp to [+1.0,-->)
        if zoomScale < 1.0:
            zoomScale = 1.0

        self.plotWindow.SetViewZoom(zoomScale)
        self.plotPropertyTab.UpdateControls()

    def GetDataSetLabel(self, dataSetId):
        return self.plotWindow.GetDataSetLabel(dataSetId)

    def SetDataSetLabel(self, dataSetId, label):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        self.plotWindow.SetDataSetLabel(dataSetId, str(label))
        self.dataSetPropertyTab.UpdateDataSetList()

    def GetOpacity(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetOpacity(dataSetId)

    def SetOpacity(self, dataSetId, opacity):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            opacity = float(opacity)
        except ValueError:
            raise TypeError("opacity property should be a float (was: '%s')" % str(opacity))

        if (opacity < 0) or (opacity > 1):
            raise ValueError("opacity property should be between 0.0 and 1.0 (was: '%s')" % str(opacity))

        self.plotWindow.SetOpacity(dataSetId, opacity)

    def GetLineWidth(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetLineWidth(dataSetId)

    def SetLineWidth(self, dataSetId, linewidth):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            linewidth = float(linewidth)
        except ValueError:
            raise TypeError("linewidth property should be a float (was: '%s')" % str(linewidth))

        if (linewidth < 0):
            raise ValueError("linewidth property should be greater than 0 (was: '%s')" % str(linewidth))

        self.plotWindow.SetLineWidth(dataSetId, linewidth)

    def GetPointSize(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetPointSize(dataSetId)

    def SetPointSize(self, dataSetId, pointsize):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            pointsize = float(pointsize)
        except ValueError:
            raise TypeError("pointsize property should be a float (was: '%s')" % str(pointsize))

        if (pointsize < 0):
            raise ValueError("pointsize property should be equal or greater than 0.0 (was: '%s')" % str(pointsize))

        self.plotWindow.SetPointSize(dataSetId, pointsize)

    def GetReferenceHeight(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetReferenceHeight(dataSetId)

    def SetReferenceHeight(self, dataSetId, referenceHeight):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            referenceHeight = float(referenceHeight)
        except ValueError:
            raise TypeError("referenceHeight property should be a float (was: '%s')" % str(referenceHeight))

        if (referenceHeight <= 0):
            raise ValueError("referenceHeight property should be greater than 0.0 (was: '%s')" % str(referenceHeight))

        self.plotWindow.SetReferenceHeight(dataSetId, referenceHeight)

    def GetHeightFactor(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetHeightFactor(dataSetId)

    def SetHeightFactor(self, dataSetId, heightFactor):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            heightFactor = float(heightFactor)
        except ValueError:
            raise TypeError("heightFactor property should be a float (was: '%s')" % str(heightFactor))

        if (heightFactor < 0):
            raise ValueError("heightFactor property should be equal or greater than 0.0 (was: '%s')" %
                             str(heightFactor))

        self.plotWindow.SetHeightFactor(dataSetId, heightFactor)

    def GetMinHeightValue(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetMinHeightValue(dataSetId)

    def SetMinHeightValue(self, dataSetId, minHeightValue):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            minHeightValue = float(minHeightValue)
        except ValueError:
            raise TypeError("minHeightValue property should be a float (was: '%s')" % str(minHeightValue))

        self.plotWindow.SetMinHeightValue(dataSetId, minHeightValue)

    def GetMaxHeightValue(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetMaxHeightValue(dataSetId)

    def SetMaxHeightValue(self, dataSetId, maxHeightValue):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            maxHeightValue = float(maxHeightValue)
        except ValueError:
            raise TypeError("maxHeightValue property should be a float (was: '%s')" % str(maxHeightValue))

        self.plotWindow.SetMaxHeightValue(dataSetId, maxHeightValue)

    def GetColorRange(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetColorRange(dataSetId)

    def SetColorRange(self, dataSetId, colorrange):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        try:
            if len(colorrange) != 2:
                raise TypeError("colorrange property should be a list of two elements")
            range = [float(colorrange[0]), float(colorrange[1])]
        except ValueError:
            raise TypeError("colorrange property should be a float array (was: '%s')" % str(colorrange))

        self.plotWindow.SetColorRange(dataSetId, range[0], range[1])
        self.dataSetPropertyTab.UpdateColorBar()

    def GetColorBarTitle(self, dataSetId):
        return str(self.plotWindow.GetColorBarTitle(dataSetId))

    def SetColorBarTitle(self, dataSetId, title):
        self.plotWindow.SetColorBarTitle(dataSetId, str(title))
        self.dataSetPropertyTab.UpdateColorBar()

    def GetNumColorBarLabels(self, dataSetId):
        return self.plotWindow.GetNumColorBarLabels(dataSetId)

    def SetNumColorBarLabels(self, dataSetId, numLabels):
        try:
            numLabels = int(numLabels)
        except ValueError:
            raise TypeError("numLabels property should be an integer (was: '%s')" % str(numLabels))

        if (numLabels < 2):
            raise ValueError("numLabels property should be equal or greater than 2 (was: '%s')" % str(numLabels))
        self.plotWindow.SetNumColorBarLabels(dataSetId, numLabels)
        self.dataSetPropertyTab.UpdateColorBar()

    def GetColorTable(self, dataSetId):
        if dataSetId < 0 or dataSetId >= self.plotWindow.GetNumDataSets():
            raise ValueError("Invalid dataSetId")
        return self.plotWindow.GetColorTable(dataSetId)

    def GetNumKeyframes(self):
        return self.plotWindow.GetNumKeyframes()

    def GetKeyframe(self):
        return self.plotWindow.GetKeyframe()

    def SetKeyframe(self, keyframe):
        self.animationToolbar.UpdateNumKeyframes()
        self.animationToolbar.SetKeyframe(keyframe)

    def OnClose(self, event):
        # Make sure this wxFrame is not closing down
        if self.closingDown:
            return

        # Fix for EVT_KILL_FOCUS crashes
        win = wx.Window.FindFocus()
        if win is not None:
            win.Disconnect(-1, -1, wx.wxEVT_KILL_FOCUS)

        self.closingDown = True
        self.animationToolbar.Destroy()
        self.Destroy()

    def OnKeyDown(self, event):
        # Check if space was pressed
        if (event.GetKeyCode() == 32):  # ' '
            self.animationToolbar.TogglePlayPause()
        elif (event.GetKeyCode() == 114):  # 'r'
            self.plotWindow.Reset()
        event.Skip()

    def OnPlotDataChanged(self, event):
        self.dataSetPropertyTab.UpdateDataSetList()
        self.animationToolbar.UpdateNumKeyframes()
        self.viewSliderMenuItem.Enable(self.plotWindow.GetNumKeyframes() > 1)

    def OnWorldViewChanged(self, event):
        self.plotPropertyTab.UpdateControls()

    def OnKeyframeChanged(self, event):
        # Make sure this wxFrame is not closing down
        if self.closingDown:
            return

        self.plotWindow.SetKeyframe(event.keyframe)
        self.dataSetPropertyTab.UpdateAttributes()

    def OnCurrentDataSetChanged(self, event):
        # Make sure this wxFrame is not closing down
        if self.closingDown:
            return
        self.plotWindow.SetDataSetForColorBar(event.dataSetId)
        self.plotPropertyTab.UpdateControls()

    def OnSplitPanelSize(self, event):
        if self.splitPanel.IsSplit():
            self.AdjustSplitPanelSashPosition()
        event.Skip()

    def OnPropertyPanelSize(self, event):
        self.propertyPanel.Layout()
        event.Skip()

    def OnDataSetPropertyTabSize(self, event):
        self.dataSetPropertyTab.Layout()
        event.Skip()

    def OnPlotPropertyTabSize(self, event):
        self.plotPropertyTab.Layout()
        event.Skip()

    def OnViewSlider(self, event):
        self.ShowAnimationToolbar(event.IsChecked())

    def OnViewProps(self, event):
        self.ShowPropertyPanel(event.IsChecked())

    def OnViewColorBar(self, event):
        self.ShowColorBar(event.IsChecked())

    def OnSave(self, event):
        imagetypeinfo = [("TIFF", "tif"), ("Windows Bitmap", "bmp"), ("JPEG", "jpg"), ("PNG", "png"), ("PNM", "pnm")]
        dialog = TypedSaveFileDialog(self, title="Save Image",
                                     initialdir=str(wx.Config.Get().Read('DirectoryLocation/Export')),
                                     typeinfo=imagetypeinfo)
        if dialog.ShowModal() == wx.ID_OK:
            wx.Config.Get().Write('DirectoryLocation/Export', os.path.dirname(dialog.filename))
            self.plotWindow.ExportToImageFile(dialog.filename, dialog.ext)



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
A collection of mathematical functions for VISAN.

  histogram()
  fit()
  lfit()
  gammap()
  gammaq()
  gammaln()
  ecef_to_wgs84()
  wgs84_to_ecef()
"""

import numpy as np
import numpy.linalg as linalg
from functools import reduce


def histogram(a, bins, mode='inner'):
    """ Create a histogram for a set of values using a specified set of bins.

    This function creates a histogram for the values in the array 'a'.
    The bins are defined by the edge values in the 'bins' array.
    The values in 'bins' should be stored in ascending order.

    If mode is 'inner' then len(bins)-1 bins will be used. If mode is 'outer'
    then two more bins are added; one for all values below bins[0] and one for
    the values in 'a' higher then bins[-1].

    Both 'a' and 'bins' should be one dimensional arrays.

    Example:

    >>> histogram([2.1, 4.5, 5, 7.8, 2.5, 6.1, 9.9, 5.1, 5.2, 7.2], arange(11))
    array([0, 2, 0, 1, 3, 1, 2, 0, 1, 0])
    """
    def accumulate(n, i):
        n[i] += 1
        return n
    a = np.asarray(a)
    bins = np.asarray(bins)
    indices = np.searchsorted(bins.flat, a.flat)
    n = reduce(accumulate, indices, np.zeros(len(bins.flat) + 1))
    if mode == 'inner':
        return n[1:-1]
    elif mode == 'outer':
        return n
    else:
        raise ValueError("Invalid value for 'mode' argument")


def fit(x, y, sigy=None, error=False):
    """ Perform a linear fit on a range of x and y values.

    This function fits a set of data points x, y with individual standard
    deviations sigy for the y values to a straight line y = a + bx by
    minimizing chi-square.

    If the 'error' parameter is False the function returns only the found
    values for 'a' and 'b' as a tuple. If 'error' is True then a third tuple
    item is returned containing a dictionary of error statistics ('siga' the
    standard deviation of 'a', 'sigb' the standard deviation of 'b', 'chi2'
    the value of chi-square, and 'q' the goodness of fit probability).

    If 'sigy' is 'None' then no standard deviations are used, 'q' will be 1.0
    and the normalization of chi-square is to unit standard deviation for all
    points.

    Example:

    >>> a, b, err = fit([1.1, 1.95, 3.05], [1, 2.01, 2.95], error=True)
    """
    # This routine is based on an algorithm from Numerical Recipes
    x = np.asarray(x).astype(float)
    y = np.asarray(y).astype(float)
    if x.size != y.size:
        ValueError("Arrays 'x' and 'y' have different length")
    if x.size < 2:
        raise ValueError("Arrays 'x' and 'y' should have at least 2 elements")
    useWeights = False
    if sigy is not None:
        sigy = np.asarray(sigy)
        if sigy.size != y.size:
            raise ValueError("Arrays 'sigy' and 'y' have different length")
        useWeights = True

    # We need to minimize:
    #
    #  chi2 = sum( ((y[i] - a - b * x[i]) / sigy[i])^2 ; i=1..N)
    #
    # When taking derivatives with respect to a and b we get
    #
    #  dchi2/da = 0 = -2 * sum( (y[i] - a - b * x[i]) / sig[i]^2 ; i=1..N)
    #  dchi2/db = 0 = -2 * sum( x[i] * (y[i] - a - b * x[i]) / sig[i]^2 ; i=1..N)
    #
    # which provides us with a linear equation that we can use to solve a and b

    if useWeights:
        weights = 1 / (np.square(sigy.flat))
        S = np.sum(weights)
        Sx = np.dot(x.flat, weights)
        Sy = np.dot(y.flat, weights)
        t = x.flat - Sx / S
        tw = t * weights
        Stt = np.dot(tw, t)
        b = np.dot(tw, y.flat) / Stt
    else:
        S = x.size
        Sx = np.sum(x.flat)
        Sy = np.sum(y.flat)
        t = x.flat - Sx / S
        Stt = np.dot(t, t)
        b = np.dot(t, y.flat) / Stt

    a = (Sy - Sx * b) / S

    if not error:
        return (a, b)

    siga = np.sqrt((1 + Sx * Sx / (S * Stt)) / S)
    sigb = np.sqrt(1 / Stt)
    chi2 = 0.0
    q = 1.0
    if useWeights:
        chi = (y - a - b * x.flat) / sigy.flat
        chi2 = np.dot(chi, chi)
        if x.size > 2:
            q = gammaq(0.5 * (x.size - 2), 0.5 * chi2)
    else:
        chi = y.flat - a - b * x.flat
        chi2 = np.dot(chi, chi)
        if x.size > 2:
            sigdat = np.sqrt(chi2 / (x.size - 2))
            siga *= sigdat
            sigb *= sigdat

    return (a, b, {'siga': siga, 'sigb': sigb, 'chi2': chi2, 'q': float(q)})


def lfit(x, y, F, sigy=None, error=False):
    """ Perform a fit on a range of values using a set of basis functions.

    This function fits a set of data points x, y with individual standard
    deviations sigy for the y values to a linear combination of basis functions
    y = sum(a_i * F_i(x); i = 1..N) minimizing chi-square.

    F should be a user provided function F(x) that returns an array where each
    element is the evaluation of the basis function F_i() at location 'x'

    The function 'lfit' will return an array 'a' with the coefficients for the
    basis functions F_i().

    If the 'error' parameter is False the function returns only the found
    values for 'a' as an array. If 'error' is True then a tuple is returned
    containing the array 'a', the covariance matrix, and the value of
    chi-square.

    If 'sigy' is 'None' then the standard deviations for all y values are set
    to 1.0.

    Example:

    >>> def F(x):
    ...    return array([1, x, x*x])
    >>> a, covar, chi2 = lfit([0, 1, 2, 3], [1, 3, 2, 1], F, error=True)
    >>> y = dot(a, F(0.5))
    """
    x = np.asarray(x).astype(float)
    y = np.asarray(y).astype(float)
    numx = np.shape(x)[0]
    if numx != y.size:
        raise ValueError("Incompatible number of 'x' and 'y' values")
    testy = F(x[0])
    if testy.ndim != 1:
        raise ValueError("Function 'F' should return a one dimensional array")
    N = testy.size
    if N > y.size:
        # We have more unknowns then measurements
        raise ValueError("Arrays 'x' and 'y' should have at least %i elements" % (testy.size))
    if sigy is not None:
        sigy = np.asarray(sigy).astype(float)
        if np.shape(sigy) != np.shape(y):
            raise ValueError("Arrays 'y' and 'sigy' should have the same shape")
    else:
        sigy = np.ones(np.shape(y), float)

    A = np.zeros((numx, N), float)
    b = np.zeros((numx), float)
    for k in range(numx):
        A[k] = F(x[k]) / sigy[k]
        b[k] = y[k] / sigy[k]
    a, chi2, rankA, svA = np.linalg.lstsq(A, b, rcond=None)

    if not error:
        return a

    covar = np.matrixmultiply(np.transpose(A), A)

    return (a, covar, chi2)


def gammap(a, x):
    """The incomplete gamma function P(a,x) (= gamma(a,x) / Gamma(a) )."""
    # This routine is based on an algorithm from Numerical Recipes
    a = np.asarray(a).astype(float)
    x = np.asarray(x).astype(float)

    if np.sometrue(x < 0):
        raise ValueError("Parameter 'x' may not contain negative value(s)")
    if np.sometrue(a <= 0):
        raise ValueError("Parameter 'a' may not have value(s) <= 0")
    if np.shape(a) != np.shape(x):
        raise ValueError("Parameters 'a' and 'x' should have the same shape")

    gp = np.zeros(np.shape(x), dtype=float)
    criterium = x.flat < np.add(a.flat, 1.0)
    serids = np.nonzero(criterium)[0]
    if serids.size > 0:
        gp.flat[serids] = _gammaBySeries(a.flat[serids], x.flat[serids])
    cfrids = np.nonzero(1 - criterium)[0]
    if cfrids.size > 0:
        gp.flat[cfrids] = 1.0 - _gammaByContinuedFractions(a.flat[cfrids], x.flat[cfrids])
    return gp


def gammaq(a, x):
    """The incomplete gamma function Q(a,x) = 1 - P(a,x) (= 1 - gamma(a,x) / Gamma(a) )."""
    # This routine is based on an algorithm from Numerical Recipes
    a = np.asarray(a).astype(float)
    x = np.asarray(x).astype(float)

    if np.sometrue(x < 0):
        raise ValueError("Parameter 'x' may not contain negative value(s)")
    if np.sometrue(a <= 0):
        raise ValueError("Parameter 'a' may not have value(s) <= 0")
    if np.shape(a) != np.shape(x):
        raise ValueError("Parameters 'a' and 'x' should have the same shape")

    gq = np.zeros(np.shape(x), dtype=float)
    criterium = x.flat < np.add(a.flat, 1.0)
    serids = np.nonzero(criterium)[0]
    if serids.size > 0:
        gq.flat[serids] = 1.0 - _gammaBySeries(a.flat[serids], x.flat[serids])
    cfrids = np.nonzero(1 - criterium)[0]
    if cfrids.size > 0:
        gq.flat[cfrids] = _gammaByContinuedFractions(a.flat[cfrids], x.flat[cfrids])
    return gq


def _gammaBySeries(a, x):
    # This routine is based on an algorithm from Numerical Recipes
    # This function assumes that 'a' and 'x' are numpy arrays with rank 1 and
    # equal lenghts
    ITMAX = 100
    EPS = 3.0e-7
    lnga = gammaln(a)
    ap = a.copy()
    d = 1.0 / a
    s = d.copy()
    for n in range(ITMAX):
        ap += 1
        d *= x / ap
        s += d
        if np.alltrue(np.fabs(d) < np.fabs(s) * EPS):
            return s * np.exp(-x + a * np.log(x) - lnga)
    raise ValueError("Parameter 'a' has value(s) that are too large")


def _gammaByContinuedFractions(a, x):
    # This routine is based on an algorithm from Numerical Recipes
    # This function assumes that 'a' and 'x' are numpy arrays with rank 1 and
    # equal lenghts
    ITMAX = 100
    EPS = 3.0e-7
    FPMIN = 1.0e-300
    lnga = gammaln(a)
    b = x + 1.0 - a
    c = 1.0 / FPMIN
    d = 1.0 / b
    h = d.copy()
    for n in range(ITMAX):
        i = n + 1
        an = -i * (i - a)
        b += 2.0
        d = an * d + b
        d[np.nonzero(np.fabs(d) < FPMIN)[0]] = FPMIN
        c = b + an / c
        c[np.nonzero(np.fabs(c) < FPMIN)[0]] = FPMIN
        d = 1.0 / d
        dl = d * c
        h *= dl
        if (np.alltrue(np.fabs(dl - 1.0) < EPS)):
            return h * np.exp(-x + a * np.log(x) - lnga)
    raise ValueError("Parameter 'a' has value(s) that are too large")


def gammaln(x):
    """Returns the value of ln(Gamma(x))."""
    # We use Lanczos approximation
    x = np.asarray(x).astype(float)
    if np.sometrue(x <= 0):
        raise ValueError("Parameter 'x' may not have value(s) <= 0")

    c = (1.000000000000000174663,
         5716.400188274341379136,
         -14815.30426768413909044,
         14291.49277657478554025,
         -6348.160217641458813289,
         1301.608286058321874105,
         -108.1767053514369634679,
         2.605696505611755827729,
         -0.7423452510201416151527e-2,
         0.5384136432509564062961e-7,
         -0.4023533141268236372067e-8)
    g = 9
    t = np.add(x.flat, g)
    s = c[0]
    for k in np.arange(g + 1, 0, -1):
        # going from g+1 to 1 with stepsize -1
        s += c[k] / t
        t -= 1
    ss = np.add(x.flat, g - 0.5)
    f = np.log(2.5066282746310005 * s) + np.add(x.flat, -0.5) * np.log(ss) - ss
    return np.reshape(f, np.shape(x))


def ecef_to_wgs84(x, y, z):
    """ Convert XYZ coordinates in Earth-Centered Earth-Fixed (ECEF) to WGS84 lat/lon/height """
    # WGS84 semi-major axis (a) and semi-minor axis (b)
    a = 6378137
    b = 6356752.31424518
    e = np.sqrt((a * a - b * b) / (a * a))
    e_prime = np.sqrt((a * a - b * b) / (b * b))
    p = np.sqrt(x * x + y * y)
    teta = np.arctan2(z * a, p * b)
    sin_teta = np.sin(teta)
    cos_teta = np.cos(teta)
    lon = np.arctan2(y, x)
    lat = np.arctan2(z + e_prime * e_prime * b * sin_teta * sin_teta * sin_teta,
                     p - e * e * a * cos_teta * cos_teta * cos_teta)
    N = a / np.sqrt(1 - e * e * np.sin(lat) * np.sin(lat))
    h = p / np.cos(lat) - N
    lat = lat * 180.0 / pi
    lon = lon * 180.0 / pi
    return lat, lon, h


def wgs84_to_ecef(lat, lon, h):
    """ Convert WGS84 lat/lon/height to XYZ coordinates in Earth-Centered Earth-Fixed (ECEF) """
    # WGS84 semi-major axis (a) and semi-minor axis (b)
    a = 6378137
    b = 6356752.31424518
    lat = lat * pi / 180.0
    lon = lon * pi / 180.0
    e = np.sqrt((a * a - b * b) / (a * a))
    N = a / np.sqrt(1 - e * e * np.sin(lat) * np.sin(lat))
    x = (N + h) * np.cos(lat) * np.cos(lon)
    y = (N + h) * np.cos(lat) * np.sin(lon)
    z = (b * b * N / (a * a) + h) * np.sin(lat)
    return x, y, z



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import wx


def _isList(obj):
    try:
        iter(obj)
    except Exception:
        return False
    else:
        try:
            import numpy
            if isinstance(obj, numpy.ndarray):
                return True
        except Exception:
            pass
        try:
            obj + ''
        except Exception:
            return True
        else:
            return False


class DataSetAttributesPanel(wx.Panel):

    def __init__(self, parent):
        panelstyle = wx.TAB_TRAVERSAL
        if wx.Platform == '__WXGTK__':
            panelstyle |= wx.SUNKEN_BORDER
        wx.Panel.__init__(self, parent, -1, style=panelstyle)

        # Create and configure all widgets
        self.CreateControls()
        self.CreateLayout()

    def CreateControls(self):
        # Create the two column list for showing attributes
        self.attributeList = wx.ListCtrl(self, -1, style=(wx.LC_REPORT | wx.LC_NO_HEADER | wx.LC_VRULES),
                                         size=(100, -1))
        self.attributeList.InsertColumn(0, "attribute")
        self.attributeList.InsertColumn(1, "value")

    def CreateLayout(self):
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.attributeList, 1, wx.EXPAND)
        self.SetSizer(sizer)

    def UpdateAttributes(self, attributes, keyframe):
        self.attributeList.DeleteAllItems()
        keys = sorted(attributes.keys())
        for key in keys:
            value = attributes[key]
            if _isList(value):
                # try to see if we can use a keyframe index for the value
                try:
                    value = value[keyframe]
                except IndexError:
                    # if the keyframe is out of range, just use the final value
                    value = value[-1]
                except Exception:
                    pass
            self.attributeList.Append([key, value])
        self.attributeList.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        if wx.Platform == '__WXMSW__':
            self.attributeList.SetColumnWidth(0, self.attributeList.GetColumnWidth(0) + 5)
        self.attributeList.SetColumnWidth(1, wx.LIST_AUTOSIZE)



# This is an example VISAN script for the GOSAT FTS L1B product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOSAT FTS L1B products.

# This example will then take the first product it finds in this directory and
# for that product plot the measured spectra for all bands


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all GOSAT TFTS L1B files in productdir
    files = glob.glob(os.path.join(productdir, "GOSATTFTS*.01"))
    if len(files) == 0:
        print(("Could not find any GOSAT FTS L1b files in directory '" + productdir + "'"))
        return

    # We only ingest data from the first file in the list
    file = files[0]

    band1p = harp.import_product(file, 'wavenumber>=12900;wavenumber<=13200', 'band=1p')
    band1s = harp.import_product(file, 'wavenumber>=12900;wavenumber<=13200', 'band=1s')
    band2p = harp.import_product(file, 'wavenumber>=5800;wavenumber<=6400', 'band=2p')
    band2s = harp.import_product(file, 'wavenumber>=5800;wavenumber<=6400', 'band=2s')
    band3p = harp.import_product(file, 'wavenumber>=4800;wavenumber<=4800', 'band=3p')
    band3s = harp.import_product(file, 'wavenumber>=4800;wavenumber<=4800', 'band=3s')
    band4 = harp.import_product(file, 'wavenumber>=650', 'band=4')

    wplot(band4, title="GOSAT FTS L1b measurement locations")

    w = plot(band1p, name="Band 1 P")
    plot(band1s, name="Band 1 S", title="GOSAT FTS Band 1 P/S", window=w)
    w = plot(band2p, name="Band 2 P")
    plot(band2s, name="Band 2 S", title="GOSAT FTS Band 2 P/S", window=w)
    w = plot(band3p, name="Band 3 P")
    plot(band3s, name="Band 3 S", title="GOSAT FTS Band 3 P/S", window=w)
    plot(band4, name="Band 4", title="GOSAT FTS Band 4")


run()



# This is an example VISAN script for the GOSAT FTS L2 CO2 product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOSAT FTS L2 CO2 products.

# This example will then take all products it finds in this directory and
# for those product plot the CO2 columns


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all GOSAT FTS L2 C01S files in productdir
    files = glob.glob(os.path.join(productdir, "GOSATTFTS*02C01S*"))
    if len(files) == 0:
        print(("Could not find any GOSAT FTS L2 C01S files in directory '" + productdir + "'"))
        return

    product = harp.import_product(files)
    wplot(product, title="GOSAT FTS L2 CO2")


run()



# This is an example VISAN script for the GOM_TRA_1P product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOM_TRA_1P products.

# This example will then take the first product it finds in this directory and
# for that product plot measured spectra


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'GOM_TRA_1P'
    files = glob.glob(os.path.join(productdir, "GOM_TRA_1P*"))
    if len(files) == 0:
        print(("Could not find any GOM_TRA_1P files in directory '" + productdir + "'"))
        return

    # We only ingest the first file from the list
    product = harp.import_product(files[0])

    plot(product, yrange=(-1, 2), points=True, pointsize=.5, lines=False, showpropertypanel=True,
         name=os.path.basename(files[0]), title="GOM_TRA_1P spectral readout example", ylabel="")


run()



# This is an example VISAN script for the histogram() function.

from visan.math import histogram


def run():

    # We will calculate a series of means
    num_means = 10000
    # Each mean will be calculated using sample set of uniformly distributed
    # values in the range [0,1]
    num_samples_per_mean = 10
    # We will use a number of bins to create a histogram showing the
    # distribution of all the means
    num_bins = 200

    # We first create all samples for all means
    values = numpy.random.random(num_means * num_samples_per_mean)
    # The we reshape the array to a matrix
    values.shape = [num_means, num_samples_per_mean]

    # Calculate the means
    means = numpy.zeros(num_means, dtype=float)
    for i in range(num_means):
        means[i] = values[i, :].mean()

    # Create the array with the edges of the bins
    bins = numpy.array(list(range(num_bins + 1)), dtype=float) / num_bins

    # Create the histogram
    h = histogram(means, bins)

    # Plot the histogram
    histogramplot(h, bins, title="Distribution of means using uniformly distributed random variables on [0,1]")


run()



# This is an example VISAN script for the IASI L2 (IASI_SND_02) product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing IASI_SND_02 products.

# This example will then take the first product it finds in this directory and
# for that product plot the total ozone columns


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all files in productdir starting with 'IASI_SND_02'
    files = glob.glob(os.path.join(productdir, "IASI_SND_02*"))

    if len(files) == 0:
        print(("Could not find any IASI_SND_02 files in directory '" + productdir + "'"))
        return

    # since the IASI files can be quite big, we just ingest data from the first file
    product = harp.import_product(files[0])

    wplot(product, title="IASI Level 2")


run()



# This is an example VISAN script for the GOSAT CAI L2 CLD product

# Make sure to set the 'products-file directory' option in the VISAN Preferences panel to
# a directory containing GOSAT CAI L2 CLD products.

# This example will then take the first product it finds in this directory and
# for that product plot the cloud flag


def run():

    import glob
    import wx

    productdir = str(wx.Config.Get().Read('DirectoryLocation/Products'))

    # Use glob to find all GOSAT CAI L2 CLDM files in productdir
    files = glob.glob(os.path.join(productdir, "GOSATTCAI*02CLDM*"))
    if len(files) == 0:
        print(("Could not find any GOSAT CAI L2 CLDM files in directory '" + productdir + "'"))
        return

    # use only the first file
    pf = coda.open(files[0])
    cloudflag = coda.fetch(pf, 'Data', 'cloudFlag', 'cloudFlag')
    latitude = coda.fetch(pf, 'Data', 'geolocation', 'latitude')
    longitude = coda.fetch(pf, 'Data', 'geolocation', 'longitude')

    # filter out all invalid values
    latitude = latitude[cloudflag != -9999]
    longitude = longitude[cloudflag != -9999]
    cloudflag = cloudflag[cloudflag != -9999]

    wplot(latitude, longitude, cloudflag, title="GOSAT CAI L2 Cloud Flag")


run()



# Copyright (C) 2002-2022 S[&]T, The Netherlands.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
from code import InteractiveInterpreter

import wx
import wx.py


class VisanShell(wx.py.shell.Shell):

    def __init__(self, parent, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.CLIP_CHILDREN,
                 introText='', locals={}, *args, **kwargs):
        self.runStartupCommands(locals)
        super(VisanShell, self).__init__(parent, id, pos, size, style, introText, locals, *args, **kwargs)
        self.setDisplayLineNumbers(True)
        if wx.Platform == '__WXMAC__':
            self.zoom(-1)

    def runStartupCommands(self, locals):
        interp = InteractiveInterpreter(locals)
        """Execute the startup commands that import default modules"""
        interp.runsource("import os")
        interp.runsource("import sys")
        interp.runsource("import wx")
        interp.runsource("import vtk")
        interp.runsource("import numpy")
        interp.runsource("np = numpy")
        interp.runsource("import coda")
        interp.runsource("import harp")
        interp.runsource("import visan")
        interp.runsource("import visan.math")
        interp.runsource("from visan.commands import *")
        self.version = interp.locals['visan'].VERSION
        self.component_versions = [
            "Python %d.%d.%d" % (sys.version_info[0], sys.version_info[1], sys.version_info[2]),
            "wxPython %s" % interp.locals['wx'].VERSION_STRING,
            "VTK %s" % interp.locals['vtk'].VTK_VERSION,
            "NumPy %s" % interp.locals['numpy'].__version__,
            "CODA %s" % interp.locals['coda'].version(),
            "HARP %s" % interp.locals['harp'].version(),
        ]

    def showIntro(self, text=''):
        if text:
            if not text.endswith(os.linesep):
                text += os.linesep
            self.write(text)
        else:
            self.write("Copyright (C) 2002-2022 S[&]T, The Netherlands.\n\n")
            self.write("Welcome to the VISAN/Python Control Shell.\n\n")
            self.write("VISAN %s (%s)\n" % (self.version, ", ".join(self.component_versions)))

    def autoCallTipShow(self, command, insertcalltip=True, forceCallTip=False):
        """Display argument spec and docstring in a popup window."""
        if self.CallTipActive():
            self.CallTipCancel()
        (name, argspec, tip) = self.interp.getCallTip(command)
        if not self.autoCallTip and not forceCallTip:
            return
        if argspec and insertcalltip:
            startpos = self.GetCurrentPos()
            self.write(argspec + ')')
            endpos = self.GetCurrentPos()
            self.SetSelection(startpos, endpos)
